# -*- coding: utf-8 -*-
from __future__ import print_function
import collections
import os
import shutil
import subprocess
import sys
from abjad.tools import stringtools
from abjad.tools import systemtools
from abjad.tools.commandlinetools.ScorePackageScript import ScorePackageScript


class ManageBuildTargetScript(ScorePackageScript):
    '''
    Manages score package build target.

    ..  shell::

        ajv target --help

    '''

    ### CLASS VARIABLES ###

    alias = 'build'
    short_description = 'Manage score package build targets.'

    paper_sizes = collections.OrderedDict([
        ('a10', ((26, 'mm'), (37, 'mm'))),
        ('a9', ((37, 'mm'), (52, 'mm'))),
        ('a8', ((52, 'mm'), (74, 'mm'))),
        ('a7', ((74, 'mm'), (105, 'mm'))),
        ('a6', ((105, 'mm'), (148, 'mm'))),
        ('a5', ((148, 'mm'), (210, 'mm'))),
        ('a4', ((210, 'mm'), (297, 'mm'))),
        ('a3', ((297, 'mm'), (420, 'mm'))),
        ('a2', ((420, 'mm'), (594, 'mm'))),
        ('a1', ((594, 'mm'), (841, 'mm'))),
        ('a0', ((841, 'mm'), (1189, 'mm'))),
        ('b10', ((31, 'mm'), (44, 'mm'))),
        ('b9', ((44, 'mm'), (62, 'mm'))),
        ('b8', ((62, 'mm'), (88, 'mm'))),
        ('b7', ((88, 'mm'), (125, 'mm'))),
        ('b6', ((125, 'mm'), (176, 'mm'))),
        ('b5', ((176, 'mm'), (250, 'mm'))),
        ('b4', ((250, 'mm'), (353, 'mm'))),
        ('b3', ((353, 'mm'), (500, 'mm'))),
        ('b2', ((500, 'mm'), (707, 'mm'))),
        ('b1', ((707, 'mm'), (1000, 'mm'))),
        ('b0', ((1000, 'mm'), (1414, 'mm'))),
        # Below are two extended sizes defined in Din 476
        ('4a0', ((1682, 'mm'), (2378, 'mm'))),
        ('2a0', ((1189, 'mm'), (1682, 'mm'))),
        # Below are ISO 269 standard C series
        ('c10', ((28, 'mm'), (40, 'mm'))),
        ('c9', ((40, 'mm'), (57, 'mm'))),
        ('c8', ((57, 'mm'), (81, 'mm'))),
        ('c7', ((81, 'mm'), (114, 'mm'))),
        ('c6', ((114, 'mm'), (162, 'mm'))),
        ('c5', ((162, 'mm'), (229, 'mm'))),
        ('c4', ((229, 'mm'), (324, 'mm'))),
        ('c3', ((324, 'mm'), (458, 'mm'))),
        ('c2', ((458, 'mm'), (648, 'mm'))),
        ('c1', ((648, 'mm'), (917, 'mm'))),
        ('c0', ((917, 'mm'), (1297, 'mm'))),
        # Below are North American paper sizes
        ('junior-legal', ((8.0, 'in'), (5.0, 'in'))),
        ('legal', ((8.5, 'in'), (14.0, 'in'))),
        ('letter', ((8.5, 'in'), (11.0, 'in'))),
        # Ledger (17x11) is a 90 degree rotation of Tabloid
        ('17x11', ((17.0, 'in'), (11.0, 'in'))),
        ('ledger', ((17.0, 'in'), (11.0, 'in'))),
        # Tabloid (11x17)
        ('11x17', ((11.0, 'in'), (17.0, 'in'))),
        ('tabloid', ((11.0, 'in'), (17.0, 'in'))),
        # government-letter by IEEE Printer Working Group),
        # for children's writing
        ('government-letter', ((8, 'in'), (10.5, 'in'))),
        ('government-legal', ((8.5, 'in'), (13.0, 'in'))),
        ('philippine-legal', ((8.5, 'in'), (13.0, 'in'))),
        # ANSI sizes
        ('ansi a', ((8.5, 'in'), (11.0, 'in'))),
        ('ansi b', ((17.0, 'in'), (11.0, 'in'))),
        ('ansi c', ((17.0, 'in'), (22.0, 'in'))),
        ('ansi d', ((22.0, 'in'), (34.0, 'in'))),
        ('ansi e', ((34.0, 'in'), (44.0, 'in'))),
        ('engineering f', ((28.0, 'in'), (40.0, 'in'))),
        # G and H are very rare, and the lengths are variable up to 90 inches
        # North American Architectural sizes
        ('arch a', ((9.0, 'in'), (12.0, 'in'))),
        ('arch b', ((12.0, 'in'), (18.0, 'in'))),
        ('arch c', ((18.0, 'in'), (24.0, 'in'))),
        ('arch d', ((24.0, 'in'), (36.0, 'in'))),
        ('arch e', ((36.0, 'in'), (48.0, 'in'))),
        ('arch e1', ((30.0, 'in'), (42.0, 'in'))),
        # Other sizes
        # Some are antique sizes which are still using in UK
        ('statement', ((5.5, 'in'), (8.5, 'in'))),
        ('half letter', ((5.5, 'in'), (8.5, 'in'))),
        ('quarto', ((8.0, 'in'), (10.0, 'in'))),
        ('octavo', ((6.75, 'in'), (10.5, 'in'))),
        ('executive', ((7.25, 'in'), (10.5, 'in'))),
        ('monarch', ((7.25, 'in'), (10.5, 'in'))),
        ('foolscap', ((8.27, 'in'), (13.0, 'in'))),
        ('folio', ((8.27, 'in'), (13.0, 'in'))),
        ('super-b', ((13.0, 'in'), (19.0, 'in'))),
        ('post', ((15.5, 'in'), (19.5, 'in'))),
        ('crown', ((15.0, 'in'), (20.0, 'in'))),
        ('large post', ((16.5, 'in'), (21.0, 'in'))),
        ('demy', ((17.5, 'in'), (22.5, 'in'))),
        ('medium', ((18.0, 'in'), (23.0, 'in'))),
        ('broadsheet', ((18.0, 'in'), (24.0, 'in'))),
        ('royal', ((20.0, 'in'), (25.0, 'in'))),
        ('elephant', ((23.0, 'in'), (28.0, 'in'))),
        ('double demy', ((22.5, 'in'), (35.0, 'in'))),
        ('quad demy', ((35.0, 'in'), (45.0, 'in'))),
        ('atlas', ((26.0, 'in'), (34.0, 'in'))),
        ('imperial', ((22.0, 'in'), (30.0, 'in'))),
        ('antiquarian', ((31.0, 'in'), (53.0, 'in'))),
        # PA4 based sizes
        ('pa0', ((840, 'mm'), (1120, 'mm'))),
        ('pa1', ((560, 'mm'), (840, 'mm'))),
        ('pa2', ((420, 'mm'), (560, 'mm'))),
        ('pa3', ((280, 'mm'), (420, 'mm'))),
        ('pa4', ((210, 'mm'), (280, 'mm'))),
        ('pa5', ((140, 'mm'), (210, 'mm'))),
        ('pa6', ((105, 'mm'), (140, 'mm'))),
        ('pa7', ((70, 'mm'), (105, 'mm'))),
        ('pa8', ((52, 'mm'), (70, 'mm'))),
        ('pa9', ((35, 'mm'), (52, 'mm'))),
        ('pa10', ((26, 'mm'), (35, 'mm'))),
        # F4 used in southeast Asia and Australia
        ('f4', ((210, 'mm'), (330, 'mm'))),
        ])

    ### PRIVATE METHODS ###

    def _cleanup_latex_ephemera(self, build_target_path):
        ephemera = [
            '.aux',
            '.log',
            '.out',
            ]
        for path in sorted(build_target_path.glob('*')):
            if path.is_file() and path.suffix in ephemera:
                path.unlink()

    def _handle_create(
        self,
        target_name,
        force,
        paper_size=None,
        orientation=None,
        ):
        from abjad import abjad_configuration
        if target_name is True:
            name = paper_size.lower().replace(' ', '-')
            name = '{}-{}'.format(name, orientation)
        else:
            name = stringtools.to_dash_case(target_name)
        dimensions = self.paper_sizes[paper_size]
        message = 'Creating build target {!r} ({}{} x {}{})'.format(
            name,
            dimensions[0][0], dimensions[0][1],
            dimensions[1][0], dimensions[1][1],
            )
        print(message)
        target_path = self._name_to_score_subdirectory_path(
            name, 'build', self._score_package_path)
        if target_path.exists() and not force:
            print('    Path exists: {!s}'.format(
                target_path.relative_to(self._score_package_path.parent)))
            sys.exit(1)
        metadata = self._read_score_metadata_json(self._score_package_path)
        metadata['score_package_name'] = self._score_package_path.name
        if orientation == 'portrait':
            rotation = 0
            width, height = self.paper_sizes[paper_size]
        else:
            rotation = -90
            height, width = self.paper_sizes[paper_size]
        metadata['rotation'] = rotation
        metadata['orientation'] = orientation
        metadata['paper_size'] = paper_size
        metadata['height'] = height[0]
        metadata['width'] = width[0]
        metadata['unit'] = height[1]
        metadata['global_staff_size'] = 12
        metadata['uppercase_composer_name'] = metadata['composer_name'].upper()
        metadata['uppercase_title'] = metadata['title'].upper()
        metadata['lilypond_version'] = \
            abjad_configuration.get_lilypond_version_string()
        source_name = 'example_build_target'
        source_path = self._get_boilerplate_path().joinpath(source_name)
        suffixes = ('.py', '.tex', '.ly', '.ily')
        for path in self._copy_tree(source_path, target_path):
            if path.is_file() and path.suffix in suffixes:
                self._template_file(path, **metadata)
        print('    Created {!s}'.format(target_path.relative_to(
            self._score_repository_path)))

    def _handle_distribute(self, build_target_name):
        print('Distributing {!r}'.format(build_target_name))
        target_names = [
            path.name for path in self._build_path.glob('*')
            if path.is_dir() and path.name not in ('assets', 'segments')
            ]
        if build_target_name not in target_names:
            print('    Invalid build target!')
            sys.exit(1)
        source_path = self._build_path.joinpath(build_target_name)
        target_path = self._distribution_path.joinpath(build_target_name)
        if target_path.exists():
            shutil.rmtree(str(target_path))
        target_path.mkdir(parents=True)
        score_path = source_path.joinpath('score.pdf')
        if score_path.exists():
            source_name = 'score.pdf'
            target_name = '{}-score.pdf'.format(build_target_name)
            shutil.copyfile(
                str(source_path.joinpath(source_name)),
                str(target_path.joinpath(target_name)),
                )
            print('    {} --> {}'.format(source_name, target_name))
        for parts_path in source_path.glob('part*.pdf'):
            source_name = parts_path.name
            target_name = '{}-{}'.format(build_target_name, source_name)
            shutil.copyfile(
                str(source_path.joinpath(source_name)),
                str(target_path.joinpath(target_name)),
                )
            print('    {} --> {}'.format(source_name, target_name))
        #archive_path = self._distribution_path.joinpath(target_name + '.zip')
        #if archive_path.exists():
        #    shutil.rmtree(str(archive_path))

    def _handle_list(self):
        paths = sorted(
            path for path in self._build_path.glob('*')
            if path.is_dir() and path.name not in ('assets', 'segments')
            )
        print('Available build targets:')
        for path in paths:
            print('    {}'.format(path.name))

    def _handle_render(
        self,
        target_name,
        render_back_cover=None,
        render_front_cover=None,
        render_music=None,
        render_parts=None,
        render_preface=None,
        render_score=None,
        ):
        build_target_path = self._build_path.joinpath(target_name)
        if not build_target_path.exists():
            print('No build target found: {!r}.'.format(target_name))
            sys.exit(1)
        paths_to_open = []
        open_score_only = False
        if not any([
            render_back_cover,
            render_front_cover,
            render_music,
            render_parts,
            render_preface,
            render_score,
            ]):
            render_back_cover = True
            render_front_cover = True
            render_music = True
            render_parts = True
            render_preface = True
            render_score = True
            open_score_only = True
        if render_music:
            path = self._render_music(build_target_path)
            paths_to_open.append(path)
        if render_parts:
            path = self._render_parts(build_target_path)
            paths_to_open.append(path)
        if render_preface:
            path = self._render_preface(build_target_path)
            paths_to_open.append(path)
        if render_front_cover:
            path = self._render_front_cover(build_target_path)
            paths_to_open.append(path)
        if render_back_cover:
            path = self._render_back_cover(build_target_path)
            paths_to_open.append(path)
        if render_score:
            path = self._render_score(build_target_path)
            paths_to_open.append(path)
        if any([
            render_preface,
            render_score,
            render_front_cover,
            render_back_cover,
            ]):
            self._cleanup_latex_ephemera(build_target_path)
        if open_score_only:
            paths_to_open = [build_target_path.joinpath('score.pdf')]
        for path in paths_to_open:
            systemtools.IOManager.open_file(str(path))

    def _process_args(self, arguments):
        self._setup_paths(arguments.score_path)
        if arguments.distribute:
            self._handle_distribute(arguments.distribute)
        if arguments.list_:
            self._handle_list()
        if arguments.new:
            self._handle_create(
                force=arguments.force,
                orientation=arguments.orientation,
                paper_size=arguments.paper_size,
                target_name=arguments.new,
                )
        if arguments.render:
            self._handle_render(
                render_back_cover=arguments.back_cover,
                render_front_cover=arguments.front_cover,
                render_music=arguments.music,
                render_parts=arguments.parts,
                render_preface=arguments.preface,
                render_score=arguments.score,
                target_name=arguments.render,
                )

    def _render_back_cover(self, build_target_path):
        path = build_target_path.joinpath('back-cover.tex')
        print('Rendering {!s}'.format(path.relative_to(self._score_package_path)))
        if not path.is_file():
            print('    Missing: {!s}'.format(path.relative_to(self._score_package_path)))
            sys.exit(1)
        return self._run_latex(path)

    def _render_front_cover(self, build_target_path):
        path = build_target_path.joinpath('front-cover.tex')
        print('Rendering {!s}'.format(path.relative_to(self._score_package_path)))
        if not path.is_file():
            print('    Missing: {!s}'.format(path.relative_to(self._score_package_path)))
            sys.exit(1)
        return self._run_latex(path)

    def _render_music(self, build_target_path):
        path = build_target_path.joinpath('music.ly')
        print('Rendering {!s}'.format(path.relative_to(self._score_package_path)))
        if not path.is_file():
            print('    Missing: {!s}'.format(path.relative_to(self._score_package_path)))
            sys.exit(1)
        return self._run_lilypond(path)

    def _render_parts(self, build_target_path):
        path = build_target_path.joinpath('parts.ly')
        print('Rendering {!s}'.format(path.relative_to(self._score_package_path)))
        if not path.is_file():
            print('    Missing: {!s}'.format(path.relative_to(self._score_package_path)))
            sys.exit(1)
        return self._run_lilypond(path)

    def _render_preface(self, build_target_path):
        path = build_target_path.joinpath('preface.tex')
        print('Rendering {!s}'.format(path.relative_to(self._score_package_path)))
        if not path.is_file():
            print('    Missing: {!s}'.format(path.relative_to(self._score_package_path)))
            sys.exit(1)
        return self._run_latex(path)

    def _render_score(self, build_target_path):
        path = build_target_path.joinpath('score.tex')
        print('Rendering {!s}'.format(path.relative_to(self._score_package_path)))
        if not path.is_file():
            print('    Missing: {!s}'.format(path.relative_to(self._score_package_path)))
            sys.exit(1)
        return self._run_latex(path)

    def _run_latex(self, latex_path):
        latex_path = latex_path.absolute()
        relative_path = latex_path.relative_to(self._score_package_path)
        command = 'xelatex {!s}'.format(latex_path)
        with systemtools.TemporaryDirectoryChange(str(latex_path.parent)):
            for _ in range(2):
                try:
                    exit_code = subprocess.call(command, shell=True)
                except:
                    print('    Failed to render: {!s}'.format(relative_path))
                    sys.exit(1)
                if exit_code:
                    print('    Failed to render: {!s}'.format(relative_path))
                    sys.exit(1)
        return latex_path.with_suffix('.pdf')

    def _run_lilypond(self, lilypond_path):
        from abjad import abjad_configuration
        command = '{} -dno-point-and-click -o {} {}'.format(
            abjad_configuration.get('lilypond_path', 'lilypond'),
            str(lilypond_path).replace('.ly', ''),
            str(lilypond_path),
            )
        with systemtools.TemporaryDirectoryChange(str(lilypond_path.parent)):
            exit_code = subprocess.call(command, shell=True)
        if exit_code:
            print('    Failed to render: {!s}'.format(
                lilypond_path.relative_to(self._score_package_path)))
            sys.exit(1)
        return lilypond_path.with_suffix('.pdf')

    def _setup_argument_parser(self, parser):
        action_group = parser.add_argument_group('actions')
        action_group = action_group.add_mutually_exclusive_group(required=True)
        action_group.add_argument(
            '--new', '-N',
            help='create a new build target',
            metavar='NAME',
            nargs='?',
            const=True,
            default=None,
            )
        action_group.add_argument(
            '--render', '-R',
            help='render sources',
            metavar='NAME',
            )
        action_group.add_argument(
            '--distribute', '-U',
            help='stage build artifacts for distribution',
            metavar='NAME',
            )
        action_group.add_argument(
            '--list', '-L',
            dest='list_',
            help='list build targets',
            action='store_true',
            )
#        action_group.add_argument(
#            '--copy', '-Y',
#            help='copy build target',
#            metavar=('SOURCE', 'TARGET'),
#            nargs=2,
#            )
#        action_group.add_argument(
#            '--rename', '-M',
#            help='rename build target',
#            metavar=('SOURCE', 'TARGET'),
#            nargs=2,
#            )
#        action_group.add_argument(
#            '--delete', '-D',
#            help='delete build target',
#            metavar='NAME',
#            )
        render_group = parser.add_argument_group(
            '--render flags',
            'Use when rendering specific assets only.',
            )
        render_group.add_argument(
            '--front-cover',
            help='render the front cover LaTeX source',
            action='store_true',
            )
        render_group.add_argument(
            '--back-cover',
            help='render the back cover LaTeX source',
            action='store_true',
            )
        render_group.add_argument(
            '--preface',
            help='render the preface LaTeX source',
            action='store_true',
            )
        render_group.add_argument(
            '--score',
            help='render the aggregate score LaTeX source',
            action='store_true',
            )
        render_group.add_argument(
            '--music',
            help='render the music LilyPond source',
            action='store_true',
            )
        render_group.add_argument(
            '--parts',
            help='render the parts LilyPond source',
            action='store_true',
            )
        create_group = parser.add_argument_group('--new options')
        create_group.add_argument(
            '--paper-size',
            metavar='PAPER_SIZE',
            help='select new build target paper size',
            choices=list(self.paper_sizes.keys()),
            default='letter',
            )
        create_group.add_argument(
            '--orientation',
            metavar='ORIENTATION',
            help='select new build target orientation',
            choices=['landscape', 'portrait'],
            default='portrait',
            )
        common_group = parser.add_argument_group('common options')
        common_group.add_argument(
            '--score-path', '-s',
            metavar='SCORE',
            help='score path or package name',
            default=os.path.curdir,
            )
        common_group.add_argument(
            '--force', '-f',
            action='store_true',
            help='force overwriting',
            )



# -*- coding: utf-8 -*-
from abjad import *


def test_rhythmmakertools_BeamSpecifier_beam_each_division_01():
    r'''Beam each cell with a multipart beam spanner.
    '''

    talea = rhythmmakertools.Talea(
        counts=[1, 1, 1, -1, 2, 2],
        denominator=32,
        )

    rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
        talea=talea,
        extra_counts_per_division=[3, 4],
        )

    divisions = [(2, 16), (5, 16)]
    selections = rhythm_maker(divisions)

    selections = Sequence(selections).flatten()
    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(selections)
    score = Score([staff])
    set_(score).autoBeaming = False

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/16
                \times 4/7 {
                    c'32 [
                    c'32
                    c'32 ]
                    r32
                    c'16 [
                    c'32 ~ ]
                }
            }
            {
                \time 5/16
                \tweak text #tuplet-number::calc-fraction-text
                \times 5/7 {
                    c'32 [
                    c'32
                    c'32
                    c'32 ]
                    r32
                    c'16 [
                    c'16
                    c'32
                    c'32
                    c'32 ]
                    r32
                    c'32
                }
            }
        }
        '''
        )



# -*- coding: utf-8 -*-
import os
import platform
from abjad.tools import commandlinetools
from abjad.tools import systemtools
from base import ScorePackageScriptTestCase
try:
    from unittest import mock
except ImportError:
    import mock


class Test(ScorePackageScriptTestCase):

    expected_files = [
        'test_score/test_score/materials/.gitignore',
        'test_score/test_score/materials/__init__.py',
        'test_score/test_score/materials/test_material/__init__.py',
        'test_score/test_score/materials/test_material/definition.py',
        'test_score/test_score/materials/test_material/illustration.ly',
        'test_score/test_score/materials/test_material/illustration.pdf',
        ]

    if platform.system().lower() == 'windows':
        expected_files = [_.replace('/', os.path.sep) for _ in expected_files]

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_success_one_material(self, open_file_mock):
        self.create_score()
        material_path = self.create_material('test_material')
        self.illustrate_material('test_material')
        pdf_path = material_path.joinpath('illustration.pdf')
        assert pdf_path.exists()
        pdf_path.unlink()
        script = commandlinetools.ManageMaterialScript()
        command = ['--render', 'test_material']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                try:
                    script(command)
                except SystemExit as e:
                    raise RuntimeError('SystemExit: {}'.format(e.code))
        self.compare_captured_output(r'''
        Rendering candidates: 'test_material' ...
        Rendering test_score/materials/test_material/
            Writing test_score/materials/test_material/illustration.pdf ... OK!
                LilyPond runtime: ... second...
            Rendered test_score/materials/test_material/
        '''.replace('/', os.path.sep))
        self.compare_path_contents(self.materials_path, self.expected_files)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_append_spacer_skips_to_underfull_measures_01():

    staff = Staff(Measure((3, 8), "c'8 d'8 e'8") * 3)
    detach(TimeSignature, staff[1])
    time_signature = TimeSignature((4, 8))
    attach(time_signature, staff[1])
    detach(TimeSignature, staff[2])
    time_signature = TimeSignature((5, 8))
    attach(time_signature, staff[2])

    assert not staff[0].is_underfull
    assert staff[1].is_underfull
    assert staff[2].is_underfull

    scoretools.append_spacer_skips_to_underfull_measures(staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/8
                c'8
                d'8
                e'8
            }
            {
                \time 4/8
                c'8
                d'8
                e'8
                s1 * 1/8
            }
            {
                \time 5/8
                c'8
                d'8
                e'8
                s1 * 1/4
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Spanner___eq___01():
    r'''Spanner is strict comparator.
    '''

    class MockSpanner(spannertools.Spanner):

        def __init__(self, components=None):
            spannertools.Spanner.__init__(self, components)

    spanner_1 = MockSpanner()
    spanner_2 = MockSpanner()

    assert not spanner_1 == spanner_2



# -*- coding: utf-8 -*-
import itertools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import mutate
from abjad.tools.selectiontools.Selection import Selection


class LogicalTie(Selection):
    r'''A selection of components in a logical tie.

    ..  container:: example

        ::

            >>> staff = Staff("c' d' e' ~ e'")
            >>> show(staff) # doctest: +SKIP

        ::

            >>> inspect_(staff[2]).get_logical_tie()
            LogicalTie([Note("e'4"), Note("e'4")])

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### PRIVATE PROPERTIES ###

    @property
    def _all_leaves_are_in_same_parent(self):
        r'''Is true when all leaves in logical tie are in same parent.

        Returns true or false.
        '''
        parents = [leaf._parent for leaf in self.leaves]
        return mathtools.all_are_equal(parents)

    @property
    def _leaves_grouped_by_immediate_parents(self):
        r'''Leaves in logical tie grouped by immediate parents of leaves.

        Returns list of lists.
        '''
        from abjad.tools import selectiontools
        result = []
        pairs_generator = itertools.groupby(self, lambda x: id(x._parent))
        for key, values_generator in pairs_generator:
            group = selectiontools.Selection(list(values_generator))
            result.append(group)
        return result

    ### PRIVATE METHODS ###

    def _add_or_remove_notes_to_achieve_written_duration(
        self, new_written_duration):
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        new_written_duration = durationtools.Duration(new_written_duration)
        if new_written_duration.is_assignable:
            self[0].written_duration = new_written_duration
            for leaf in self[1:]:
                parent = leaf._parent
                if parent:
                    index = parent.index(leaf)
                    del(parent[index])
            first = self[0]
            for spanner in first._get_spanners(spannertools.Tie):
                spanner._sever_all_components()
            #detach(spannertools.Tie, first)
        elif new_written_duration.has_power_of_two_denominator:
            durations = scoretools.make_notes(0, [new_written_duration])
            for leaf, token in zip(self, durations):
                leaf.written_duration = token.written_duration
            if len(self) == len(durations):
                pass
            elif len(durations) < len(self):
                for leaf in self[len(durations):]:
                    parent = leaf._parent
                    if parent:
                        index = parent.index(leaf)
                        del(parent[index])
            elif len(self) < len(durations):
                for spanner in self[0]._get_spanners(spannertools.Tie):
                    spanner._sever_all_components()
                #detach(spannertools.Tie, self[0])
                difference = len(durations) - len(self)
                extra_leaves = self[0] * difference
                for extra_leaf in extra_leaves:
                    for spanner in extra_leaf._get_spanners():
                        spanner._remove(extra_leaf)
                extra_tokens = durations[len(self):]
                for leaf, token in zip(extra_leaves, extra_tokens):
                    leaf.written_duration = token.written_duration
                ties = self[-1]._get_spanners(spannertools.Tie)
                if not ties:
                    tie = spannertools.Tie()
                    if all(tie._attachment_test(_) for _ in self):
                        attach(tie, list(self))
                self[-1]._splice(extra_leaves, grow_spanners=True)
        else:
            durations = scoretools.make_notes(0, new_written_duration)
            assert isinstance(durations[0], scoretools.Tuplet)
            fmtuplet = durations[0]
            new_logical_tie_written = \
                fmtuplet[0]._get_logical_tie()._preprolated_duration
            self._add_or_remove_notes_to_achieve_written_duration(
                new_logical_tie_written)
            multiplier = fmtuplet.multiplier
            scoretools.Tuplet(multiplier, self.leaves)
        return self[0]._get_logical_tie()

    def _fuse_leaves_by_immediate_parent(self):
        result = []
        parts = self._leaves_grouped_by_immediate_parents
        for part in parts:
            result.append(part._fuse())
        return result

    def _scale(self, multiplier):
        new_duration = multiplier * self.written_duration
        return self._add_or_remove_notes_to_achieve_written_duration(
            new_duration)

    ### PUBLIC PROPERTIES ###

    @property
    def head(self):
        r'''Reference to element ``0`` in logical tie.

        Returns component.
        '''
        if self._music:
            return self._music[0]

    @property
    def is_pitched(self):
        r'''Is true when logical tie head is a note or chord.

        Returns true or false.
        '''
        from abjad.tools import scoretools
        return isinstance(self.head, (scoretools.Note, scoretools.Chord))

    @property
    def is_trivial(self):
        r'''Is true when length of logical tie is less than or equal to ``1``.

        Returns true or false.
        '''
        return len(self) <= 1

    @property
    def leaves(self):
        r'''Leaves in logical tie.

        Returns tuple.
        '''
        from abjad.tools import spannertools
        prototype = (spannertools.Tie,)
        try:
            tie_spanner = self[0]._get_spanner(prototype=prototype)
            return tuple(tie_spanner._get_leaves())
        except MissingSpannerError:
            assert self.is_trivial
            return (self[0], )

    @property
    def tail(self):
        r'''Reference to element ``-1`` in logical tie.

        Returns component.
        '''
        if self._music:
            return self._music[-1]

    @property
    def tie_spanner(self):
        r'''Tie spanner governing logical tie.

        Returns tie spanner.
        '''
        from abjad.tools import spannertools
        if 1 < len(self):
            prototype = (spannertools.Tie,)
            for component in self[0].parentage:
                try:
                    tie_spanner = component._get_spanner(prototype)
                    break
                except MissingSpannerError:
                    pass
            return tie_spanner

    @property
    def written_duration(self):
        r'''Sum of written duration of all components in logical tie.

        Returns duration.
        '''
        return sum([x.written_duration for x in self])

    ### PUBLIC METHODS ###

    def to_tuplet(
        self,
        proportions,
        dotted=False,
        is_diminution=True,
        ):
        r'''Change logical tie to tuplet.

        ..  container:: example

            **Example 1.** Change logical tie to diminished tuplet:

            ::

                >>> staff = Staff(r"c'8 ~ c'16 cqs''4")
                >>> crescendo = spannertools.Hairpin(descriptor='p < f')
                >>> attach(crescendo, staff[:])
                >>> override(staff).dynamic_line_spanner.staff_padding = 3
                >>> time_signature = TimeSignature((7, 16))
                >>> attach(time_signature, staff)

            ..  doctest::

                >>> print(format(staff))
                \new Staff \with {
                    \override DynamicLineSpanner.staff-padding = #3
                } {
                    \time 7/16
                    c'8 ~ \< \p
                    c'16
                    cqs''4 \f
                }

            ::

                >>> show(staff) # doctest: +SKIP

            ::

                >>> logical_tie = inspect_(staff[0]).get_logical_tie()
                >>> logical_tie.to_tuplet([2, 1, 1, 1], is_diminution=True)
                FixedDurationTuplet(Duration(3, 16), "c'8 c'16 c'16 c'16")

            ..  doctest::

                >>> print(format(staff))
                \new Staff \with {
                    \override DynamicLineSpanner.staff-padding = #3
                } {
                    \time 7/16
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 3/5 {
                        c'8 \< \p
                        c'16
                        c'16
                        c'16
                    }
                    cqs''4 \f
                }

            ::

                >>> show(staff) # doctest: +SKIP

        ..  container:: example

            **Example 2.** Change logical tie to augmented tuplet:

            ::

                >>> staff = Staff(r"c'8 ~ c'16 cqs''4")
                >>> crescendo = spannertools.Hairpin(descriptor='p < f')
                >>> attach(crescendo, staff[:])
                >>> override(staff).dynamic_line_spanner.staff_padding = 3
                >>> time_signature = TimeSignature((7, 16))
                >>> attach(time_signature, staff)

            ..  doctest::

                >>> print(format(staff))
                \new Staff \with {
                    \override DynamicLineSpanner.staff-padding = #3
                } {
                    \time 7/16
                    c'8 ~ \< \p
                    c'16
                    cqs''4 \f
                }

            ::

                >>> show(staff) # doctest: +SKIP

            ::

                >>> logical_tie = inspect_(staff[0]).get_logical_tie()
                >>> logical_tie.to_tuplet([2, 1, 1, 1], is_diminution=False)
                FixedDurationTuplet(Duration(3, 16), "c'16 c'32 c'32 c'32")

            ..  doctest::

                >>> print(format(staff))
                \new Staff \with {
                    \override DynamicLineSpanner.staff-padding = #3
                } {
                    \time 7/16
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 6/5 {
                        c'16 \< \p
                        c'32
                        c'32
                        c'32
                    }
                    cqs''4 \f
                }

            ::

                >>> show(staff) # doctest: +SKIP

        Returns tuplet.
        '''
        from abjad.tools import mathtools
        from abjad.tools import scoretools
        from abjad.tools import spannertools

        # coerce input
        proportions = mathtools.Ratio(proportions)

        # find target duration of fixed-duration tuplet
        target_duration = self._preprolated_duration

        # find duration of each note in tuplet
        prolated_duration = target_duration / sum(proportions.numbers)

        # find written duration of each notes in tuplet
        if is_diminution:
            if dotted:
                basic_written_duration = \
                    prolated_duration.equal_or_greater_assignable
            else:
                basic_written_duration = \
                    prolated_duration.equal_or_greater_power_of_two
        else:
            if dotted:
                basic_written_duration = \
                    prolated_duration.equal_or_lesser_assignable
            else:
                basic_written_duration = \
                    prolated_duration.equal_or_lesser_power_of_two

        # find written duration of each note in tuplet
        written_durations = [
            _ * basic_written_duration for _ in proportions.numbers
            ]

        # make tuplet notes
        try:
            notes = [scoretools.Note(0, x) for x in written_durations]
        except AssignabilityError:
            denominator = target_duration._denominator
            note_durations = [
                durationtools.Duration(_, denominator)
                for _ in proportions.numbers
                ]
            notes = scoretools.make_notes(0, note_durations)

        # make tuplet
        tuplet = scoretools.FixedDurationTuplet(target_duration, notes)

        # replace logical tie with tuplet
        mutate(self).replace(tuplet)

        # untie tuplet
        for spanner in tuplet._get_spanners(spannertools.Tie):
            spanner._sever_all_components()
        #detach(spannertools.Tie, tuplet)

        # return tuplet
        return tuplet



# -*- coding: utf-8 -*-
from abjad.tools import abctools


class ImageLayoutSpecifier(abctools.AbjadValueObject):
    r'''An image layout specifier.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Internals'

    __slots__ = (
        '_pages',
        '_with_columns',
        '_with_thumbnail',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        pages=None,
        with_columns=None,
        with_thumbnail=None,
        ):
        self._pages = pages or None
        if with_columns is not None:
            with_columns = int(with_columns)
            if with_columns < 1:
                with_columns = None
        self._with_columns = with_columns or None
        self._with_thumbnail = with_thumbnail or None

    ### PUBLIC METHODS ###

    @classmethod
    def from_options(cls, **options):
        r'''Creates image specifier from `options` dictionary.

        Returns image specifier.
        '''
        pages = options.get('pages', None) or None
        with_columns = options.get('with_columns', None) or None
        with_thumbnail = options.get('with_thumbnail', None) or None
        if all(_ is None for _ in (
            pages,
            with_columns,
            with_thumbnail,
            )):
            return None
        return cls(
            pages=pages,
            with_columns=with_columns,
            with_thumbnail=with_thumbnail,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def pages(self):
        r'''Gets page indices.

        Returns tuple of integers or none.
        '''
        return self._pages

    @property
    def with_columns(self):
        r'''Gets column count for table layout.

        Return integer or none.
        '''
        return self._with_columns

    @property
    def with_thumbnail(self):
        r'''Is true if image should have a thumbnail.

        Returns true or false.
        '''
        return self._with_thumbnail



# -*- coding: utf-8 -*-
import collections


def get_shared_numeric_sign(argument):
    r'''Gets shared numeric sign of items in `argument`.

    ..  container:: example

        Returns ``1`` when all `argument` items are positive:

        ::

            >>> mathtools.get_shared_numeric_sign([1, 2, 3])
            1

        Returns ``-1`` when all `argument` items are negative:

        ::

            >>> mathtools.get_shared_numeric_sign([-1, -2, -3])
            -1

        Returns ``0`` on empty `argument`:

        ::

            >>> mathtools.get_shared_numeric_sign([])
            0

        Otherwise returns none:

        ::

            >>> mathtools.get_shared_numeric_sign([1, 2, -3]) is None
            True

    Returns ``1``, ``-1``, ``0`` or none.
    '''
    if not isinstance(argument, collections.Iterable):
        raise TypeError(argument)
    if len(argument) == 0:
        return 0
    elif all(0 < x for x in argument):
        return 1
    elif all(x < 0 for x in argument):
        return -1
    else:
        return None



# -*- coding: utf-8 -*-
from abjad import *


def test_timespantools_Timespan___or___01():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, -5)
    result = timespan_1 | timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(-10, -5),
        timespantools.Timespan(0, 15)
    ])

def test_timespantools_Timespan___or___02():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, 0)
    result = timespan_1 | timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(-10, 15)
    ])

def test_timespantools_Timespan___or___03():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, 5)
    result = timespan_1 | timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(-10, 15)
    ])

def test_timespantools_Timespan___or___04():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, 15)
    result = timespan_1 | timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(-10, 15)
    ])

def test_timespantools_Timespan___or___05():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, 25)
    result = timespan_1 | timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(-10, 25)
    ])

def test_timespantools_Timespan___or___06():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(0, 10)
    result = timespan_1 | timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(0, 15)
    ])

def test_timespantools_Timespan___or___07():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(0, 15)
    result = timespan_1 | timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(0, 15)
    ])

def test_timespantools_Timespan___or___08():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(5, 10)
    result = timespan_1 | timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(0, 15)
    ])

def test_timespantools_Timespan___or___09():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(5, 15)
    result = timespan_1 | timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(0, 15)
    ])

def test_timespantools_Timespan___or___10():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(0, 25)
    result = timespan_1 | timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(0, 25)
    ])

def test_timespantools_Timespan___or___11():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(5, 25)
    result = timespan_1 | timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(0, 25)
    ])

def test_timespantools_Timespan___or___12():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(15, 25)
    result = timespan_1 | timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(0, 25)
    ])

def test_timespantools_Timespan___or___13():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(20, 25)
    result = timespan_1 | timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(0, 15),
        timespantools.Timespan(20, 25)
    ])



# -*- coding: utf-8 -*-
from abjad.tools import pitchtools
from abjad.tools import sequencetools
from abjad.tools import stringtools
from abjad.tools.pitchtools import IntervalSegment


class RootlessChordClass(IntervalSegment):
    r'''A rootless chord class.

    ..  container:: example

        Major triad in root position:

        ::

            >>> tonalanalysistools.RootlessChordClass('major')
            MajorTriadInRootPosition('P1', '+M3', '+P5')

    ..  container:: example

        Dominant seventh in root position:

        ::

            >>> tonalanalysistools.RootlessChordClass('dominant', 7)
            DominantSeventhInRootPosition('P1', '+M3', '+P5', '+m7')

    ..  container:: example

        German augmented sixth in root position:

        >>> tonalanalysistools.RootlessChordClass('German', 'augmented sixth')
        GermanAugmentedSixthInRootPosition('P1', '+M3', '+m3', '+aug2')

    '''

    ### CLASS VARIABLES ###

    _segment_to_quality_and_extent = {
        '<+m3, +m3>': ('diminished', 5),
        '<+m3, +M3>': ('minor', 5),
        '<+M3, +m3>': ('major', 5),
        '<+M3, +M3>': ('augmented', 5),
        '<+M3, M2, +M3>': ('augmented French', 6),
        '<+M3, +m3, +2>': ('augmented German', 6),
        '<+M3, P1, +4>': ('augmented Italian', 6),
        '<+M3, +2, +m3>': ('augmented Swiss', 6),
        '<+m3, +m3, +m3>': ('diminished', 7),
        '<+m3, +m3, +M3>': ('half diminished', 7),
        '<+m3, +M3, +m3>': ('minor', 7),
        '<+M3, +m3, +m3>': ('dominant', 7),
        '<+M3, +m3, +M3>': ('major', 7),
        '<+M3, +m3, +m3, +M3>': ('dominant', 9),
        }

    __slots__ = (
        '_quality_string',
        '_rotation',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        quality_string='major',
        extent='triad',
        inversion='root',
        ):
        if extent in ('triad', 5):
            intervals = self._initialize_triad(quality_string)
        elif extent in ('seventh', 7):
            intervals = self._initialize_seventh(quality_string)
        elif extent in ('ninth', 9):
            intervals = self._initialize_ninth(quality_string)
        elif extent in ('augmented sixth', 6):
            intervals = self._initialize_augmented_sixth(quality_string)
        else:
            message = 'unknown chord quality arguments.'
            raise ValueError(message)
        intervals, rotation = self._invert_chord_quality(
            intervals, inversion)
        IntervalSegment.__init__(
            self,
            items=intervals,
            item_class=pitchtools.NamedInterval,
            )
        self._quality_string = quality_string
        self._rotation = rotation

    ### SPECIAL METHODS ###

    def __repr__(self):
        r'''Gets interpreter representation of rootless chord-class.

        Returns string.
        '''
        parts = []
        if self.item_class.__name__.startswith('Named'):
            parts = [repr(str(x)) for x in self]
        else:
            parts = [str(x) for x in self]
        return '{}({})'.format(
            self._title_case_name,
            ', '.join(parts),
            )

    ### PRIVATE METHODS ###

    @staticmethod
    def _initialize_augmented_sixth(quality_string):
        if quality_string == 'French':
            intervals = [
                pitchtools.NamedInterval('major', 3),
                pitchtools.NamedInterval('major', 2),
                pitchtools.NamedInterval('major', 3),
                ]
        elif quality_string == 'German':
            intervals = [
                pitchtools.NamedInterval('major', 3),
                pitchtools.NamedInterval('minor', 3),
                pitchtools.NamedInterval('augmented', 2),
                ]
        elif quality_string == 'Italian':
            intervals = [
                pitchtools.NamedInterval('major', 3),
                pitchtools.NamedInterval('perfect', 1),
                pitchtools.NamedInterval('augmented', 4),
                ]
        elif quality_string == 'Swiss':
            intervals = [
                pitchtools.NamedInterval('major', 3),
                pitchtools.NamedInterval('augmented', 2),
                pitchtools.NamedInterval('minor', 3),
                ]
        else:
            message = 'unaccpetable quality string.'
            raise ValueError(message)
        intervals.insert(0, pitchtools.NamedInterval('perfect', 1))
        return intervals

    @staticmethod
    def _initialize_ninth(quality_string):
        if quality_string == 'dominant':
            intervals = [
                pitchtools.NamedInterval('major', 3),
                pitchtools.NamedInterval('perfect', 5),
                pitchtools.NamedInterval('minor', 7),
                pitchtools.NamedInterval('major', 9),
                ]
        else:
            message = 'unacceptable quality string.'
            raise ValueError(message)
        intervals.insert(0, pitchtools.NamedInterval('perfect', 1))
        return intervals

    @staticmethod
    def _initialize_seventh(quality_string):
        if quality_string == 'dominant':
            intervals = [
                pitchtools.NamedInterval('major', 3),
                pitchtools.NamedInterval('perfect', 5),
                pitchtools.NamedInterval('minor', 7),
                ]
        elif quality_string == 'major':
            intervals = [
                pitchtools.NamedInterval('major', 3),
                pitchtools.NamedInterval('perfect', 5),
                pitchtools.NamedInterval('major', 7),
                ]
        elif quality_string == 'minor':
            intervals = [
                pitchtools.NamedInterval('minor', 3),
                pitchtools.NamedInterval('perfect', 5),
                pitchtools.NamedInterval('minor', 7),
                ]
        elif quality_string in ('diminished', 'fully diminished'):
            intervals = [
                pitchtools.NamedInterval('minor', 3),
                pitchtools.NamedInterval('diminished', 5),
                pitchtools.NamedInterval('diminished', 7),
                ]
        elif quality_string == 'half diminished':
            intervals = [
                pitchtools.NamedInterval('minor', 3),
                pitchtools.NamedInterval('perfect', 5),
                pitchtools.NamedInterval('diminished', 7),
                ]
        else:
           message = 'unaccpetable quality string.'
           raise ValueError(message)
        intervals.insert(0, pitchtools.NamedInterval('perfect', 1))
        return intervals

    @staticmethod
    def _initialize_triad(quality_string):
        if quality_string == 'major':
            intervals = [
                pitchtools.NamedInterval('major', 3),
                pitchtools.NamedInterval('perfect', 5),
                ]
        elif quality_string == 'minor':
            intervals = [
                pitchtools.NamedInterval('minor', 3),
                pitchtools.NamedInterval('perfect', 5),
                ]
        elif quality_string == 'diminished':
            intervals = [
                pitchtools.NamedInterval('minor', 3),
                pitchtools.NamedInterval('diminished', 5),
                ]
        elif quality_string == 'augmented':
            intervals = [
                pitchtools.NamedInterval('major', 3),
                pitchtools.NamedInterval('augmented', 5),
                ]
        else:
            message = 'unacceptable quality string: {!r}.'
            message = message.format(quality_string)
            raise ValueError(message)
        intervals.insert(0, pitchtools.NamedInterval('perfect', 1))
        return intervals

    @staticmethod
    def _invert_chord_quality(intervals, inversion):
        if isinstance(inversion, int):
            intervals = sequencetools.Sequence(intervals).rotate(n=-inversion)
            rotation = -inversion
        elif inversion == 'root':
            rotation = 0
        elif inversion == 'first':
            intervals = sequencetools.Sequence(intervals).rotate(n=-1)
            rotation = -1
        elif inversion == 'second':
            intervals = sequencetools.Sequence(intervals).rotate(n-2)
            rotation = -2
        elif inversion == 'third':
            intervals = sequencetools.Sequence(intervals).rotate(n=-3)
            rotation = -3
        elif inversion == 'fourth':
            intervals = sequencetools.Sequence(intervals).rotate(n=-4)
            rotation = -4
        else:
            message = 'unknown chord inversion: {!r}.'
            raise ValueError(message.format(inversion))
        return intervals, rotation

    ### PUBLIC METHODS ###

    @staticmethod
    def from_interval_class_segment(segment):
        r'''Makes new rootless chord-class from `segment`.

        Returns new rootless chord-class.
        '''
        quality, extent = \
            RootlessChordClass._segment_to_quality_and_extent[str(segment)]
        return RootlessChordClass(quality, extent=extent)

    ### PRIVATE PROPERTIES ###

    @property
    def _acceptable_augmented_sixth_qualities(self):
        return (
            'french',
            'german',
            'italian',
            'swiss',
            )

    @property
    def _acceptable_ninth_qualities(self):
        return (
            'dominant',
            )

    @property
    def _acceptable_seventh_qualities(self):
        return (
            'dominant',
            'major',
            'minor',
            'fully diminshed',
            'half diminished',
            )

    @property
    def _acceptable_triad_qualities(self):
        return (
            'major',
            'minor',
            'diminished',
            'augmented',
            )

    @property
    def _title_case_name(self):
        return '{}{}In{}'.format(
            stringtools.to_upper_camel_case(self.quality_string),
            stringtools.to_upper_camel_case(self.extent_name),
            stringtools.to_upper_camel_case(self.position),
            )

    ### PUBLIC PROPERTIES ###

    @property
    def cardinality(self):
        r'''Cardinality of rootless chord-class.

        Returns nonnegative integer.
        '''
        return len(self)

    @property
    def extent(self):
        r'''Extent of rootless chord-class.

        Returns nonnegative integer.
        '''
        from abjad.tools import tonalanalysistools
        return tonalanalysistools.RootedChordClass.cardinality_to_extent(self.cardinality)

    @property
    def extent_name(self):
        r'''Extent name of rootless chord class.
        '''
        from abjad.tools import tonalanalysistools
        if self._quality_string.lower() in \
            self._acceptable_augmented_sixth_qualities:
            return 'augmented sixth'
        return tonalanalysistools.RootedChordClass.extent_to_extent_name(self.extent)

    @property
    def inversion(self):
        r'''Inversion of rootless chord-class.

        Returns nonnegative integer.
        '''
        return abs(self.rotation)

    @property
    def position(self):
        r'''Position of rootless chord-class.

        Returns string.
        '''
        if self.rotation == 0:
            return 'root position'
        elif self.rotation == -1:
            return 'first inversion'
        elif self.rotation == -2:
            return 'second inversion'
        elif self.rotation == -3:
            return 'third inversion'
        elif self.rotation == -4:
            return 'fourth inversion'
        else:
            message = 'unknown chord position: {!r}.'
            message = message.format(self)
            raise ValueError(message)

    @property
    def quality_string(self):
        r'''Quality string of rootless chord class.

        Returns string.
        '''
        return self._quality_string

    @property
    def rotation(self):
        r'''Rotation of rootless chord-class.

        Returns nonnegative integer.
        '''
        return self._rotation



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_scale_measure_denominator_and_adjust_measure_contents_01():
    r'''Makes measure with power-of-two denominator into equivalent
    measure with non-power-of-two denominator.
    Assignable 3/2 multiplier conserves note-heads.
    '''

    measure = Measure((2, 8), "c'8 d'8")
    beam = Beam()
    attach(beam, measure[:])

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 2/8
            c'8 [
            d'8 ]
        }
        '''
        )

    scoretools.scale_measure_denominator_and_adjust_measure_contents(
        measure, 3)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 3/12
            \scaleDurations #'(2 . 3) {
                c'8. [
                d'8. ]
            }
        }
        '''
        )

    assert inspect_(measure).is_well_formed()


def test_scoretools_scale_measure_denominator_and_adjust_measure_contents_02():
    r'''Make measure with power-of-two denominator into equivalent
    measure with non-power-of-two denominator.
    Nonassignable 5/4 multiplier induces ties.
    '''

    measure = Measure((2, 8), "c'8 d'8")
    beam = Beam()
    attach(beam, measure[:])

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 2/8
            c'8 [
            d'8 ]
        }
        '''
        )

    scoretools.scale_measure_denominator_and_adjust_measure_contents(
        measure, 5)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 5/20
            \scaleDurations #'(4 . 5) {
                c'8 ~ [
                c'32
                d'8 ~
                d'32 ]
            }
        }
        '''
        )

    assert inspect_(measure).is_well_formed()


def test_scoretools_scale_measure_denominator_and_adjust_measure_contents_03():
    r'''Make measure with power-of-two denominatorinto equivalent
    measure with non-power-of-two denominator.
    Assignable 7/4 multiplier conserves note_heads.
    '''

    measure = Measure((2, 8), "c'8 d'8")
    beam = Beam()
    attach(beam, measure[:])

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 2/8
            c'8 [
            d'8 ]
        }
        '''
        )

    scoretools.scale_measure_denominator_and_adjust_measure_contents(
        measure, 7)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 7/28
            \scaleDurations #'(4 . 7) {
                c'8.. [
                d'8.. ]
            }
        }
        '''
        )

    assert inspect_(measure).is_well_formed()


def test_scoretools_scale_measure_denominator_and_adjust_measure_contents_04():
    r'''Make measure with power-of-two denominatorinto equivalent
    measure with non-power-of-two denominator.
    Nonassignable 9/8 multiplier induces ties.
    '''

    measure = Measure((2, 8), "c'8 d'8")
    beam = Beam()
    attach(beam, measure[:])

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 2/8
            c'8 [
            d'8 ]
        }
        '''
        )

    scoretools.scale_measure_denominator_and_adjust_measure_contents(
        measure, 9)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 9/36
            \scaleDurations #'(8 . 9) {
                c'8 ~ [
                c'64
                d'8 ~
                d'64 ]
            }
        }
        '''
        )

    assert inspect_(measure).is_well_formed()



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class ChordExtent(AbjadObject):
    '''A chord extent, such as triad, seventh chord, ninth chord, etc.

    Value object that can not be changed after instantiation.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_number',
        )

    _acceptable_number = (
        5,
        7,
        9,
        )

    _extent_number_to_extent_name = {
        5: 'triad',
        7: 'seventh',
        9: 'ninth',
        }

    ### INITIALIZER ###

    def __init__(self, number=5):
        if isinstance(number, int):
            if number not in self._acceptable_number:
                message = 'can not initialize extent: {}.'
                raise ValueError(message.format(number))
            number = number
        elif isinstance(number, type(self)):
            number = number.number
        self._number = number

    ### SPECIAL METHODS ###

    def __eq__(self, arg):
        r'''Is true when `arg` is a chord extent with number equal to that of
        this chord extent. Otherwise false.

        Returns true or false.
        '''
        if isinstance(arg, type(self)):
            if self.number == arg.number:
                return True
        return False

    def __hash__(self):
        r'''Hashes chord extent.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(ChordExtent, self).__hash__()

    def __ne__(self, arg):
        r'''Is true when chord extent does not equal `arg`. Otherwise false.

        Returns true or false.
        '''
        return not self == arg

    ### PUBLIC PROPERTIES ###

    @property
    def name(self):
        r'''Name of chord extent.

        Returns string.
        '''
        return self._extent_number_to_extent_name[self.number]

    @property
    def number(self):
        r'''Number of chord extent.

        Returns nonnegative integer.
        '''
        return self._number



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_agenttools_MutationAgent_fuse_01():
    r'''Works with list of leaves.
    '''

    notes = 8 * Note("c'4")
    fused = mutate(notes).fuse()

    assert len(fused) == 1
    assert fused[0].written_duration == Duration(2)


def test_agenttools_MutationAgent_fuse_02():
    r'''Works with Leaf component.
    '''

    fused = mutate(Note("c'4")).fuse()
    assert len(fused) == 1
    assert fused[0].written_duration == Duration(1, 4)


def test_agenttools_MutationAgent_fuse_03():
    r'''Works with containers.
    '''

    voice = Voice(8 * Note("c'4"))
    fused = mutate(voice[:]).fuse()
    assert len(fused) == 1
    assert fused[0].written_duration == 2
    assert voice[0] is fused[0]


def test_agenttools_MutationAgent_fuse_04():
    r'''Fusion results in tied notes.
    '''

    voice = Voice([Note(0, (2, 16)), Note(9, (3, 16))])
    fused = mutate(voice[:]).fuse()

    assert len(fused) == 2
    assert fused[0].written_duration == Duration(1, 4)
    assert fused[1].written_duration == Duration(1, 16)

    tie_1 = inspect_(fused[0]).get_spanner(Tie)
    tie_2 = inspect_(fused[1]).get_spanner(Tie)

    assert tie_1 is tie_2
    assert voice[0] is fused[0]
    assert voice[1] is fused[1]
    assert voice[0].written_pitch == voice[1].written_pitch


def test_agenttools_MutationAgent_fuse_05():
    r'''Fuses leaves with differing LilyPond multipliers.
    '''

    staff = Staff([scoretools.Skip((1, 1)), scoretools.Skip((1, 1))])
    attach(Multiplier(1, 16), staff[0])
    attach(Multiplier(5, 16), staff[1])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            s1 * 1/16
            s1 * 5/16
        }
        '''
        )

    assert inspect_(staff).get_duration() == Duration(3, 8)

    mutate(staff[:]).fuse()

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            s1 * 3/8
        }
        '''
        )

    assert inspect_(staff).get_duration() == Duration(3, 8)
    assert inspect_(staff).is_well_formed()


def test_agenttools_MutationAgent_fuse_06():
    r'''Fuses two unincorporated fixed-duration tuplets with same multiplier.
    '''

    tuplet_1 = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    beam = Beam()
    attach(beam, tuplet_1[:])
    tuplet_2 = scoretools.FixedDurationTuplet((2, 16), "c'16 d'16 e'16")
    slur = Slur()
    attach(slur, tuplet_2[:])

    assert format(tuplet_1) == stringtools.normalize(
        r'''
        \times 2/3 {
            c'8 [
            d'8
            e'8 ]
        }
        '''
        )

    assert format(tuplet_2) == stringtools.normalize(
        r'''
        \times 2/3 {
            c'16 (
            d'16
            e'16 )
        }
        '''
        )

    tuplets = select([tuplet_1, tuplet_2])
    new = mutate(tuplets).fuse()

    assert format(new) == stringtools.normalize(
        r'''
        \times 2/3 {
            c'8 [
            d'8
            e'8 ]
            c'16 (
            d'16
            e'16 )
        }
        '''
        )

    assert len(tuplet_1) == 0
    assert len(tuplet_2) == 0
    assert new is not tuplet_1 and new is not tuplet_2
    assert inspect_(new).is_well_formed()


def test_agenttools_MutationAgent_fuse_07():
    r'''Fuses fixed-duration tuplets with same multiplier in score.
    '''

    tuplet_1 = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    beam = Beam()
    attach(beam, tuplet_1[:])
    tuplet_2 = scoretools.FixedDurationTuplet((2, 16), "c'16 d'16 e'16")
    slur = Slur()
    attach(slur, tuplet_2[:])
    voice = Voice([tuplet_1, tuplet_2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \times 2/3 {
                c'8 [
                d'8
                e'8 ]
            }
            \times 2/3 {
                c'16 (
                d'16
                e'16 )
            }
        }
        '''
        )

    tuplets = voice[:]
    mutate(tuplets).fuse()

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \times 2/3 {
                c'8 [
                d'8
                e'8 ]
                c'16 (
                d'16
                e'16 )
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_fuse_08():
    r'''Fuses fixed-multiplier tuplets with same multiplier in score.
    '''

    tuplet_1 = Tuplet(Multiplier(2, 3), "c'8 d'8 e'8")
    beam = Beam()
    attach(beam, tuplet_1[:])
    tuplet_2 = Tuplet(Multiplier(2, 3), "c'8 d'8 e'8 f'8 g'8")
    slur = Slur()
    attach(slur, tuplet_2[:])
    voice = Voice([tuplet_1, tuplet_2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \times 2/3 {
                c'8 [
                d'8
                e'8 ]
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                c'8 (
                d'8
                e'8
                f'8
                g'8 )
            }
        }
        '''
        )

    tuplets = voice[:]
    mutate(tuplets).fuse()

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                c'8 [
                d'8
                e'8 ]
                c'8 (
                d'8
                e'8
                f'8
                g'8 )
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_fuse_09():
    r'''Tuplets must carry same multiplier.
    '''

    tuplet_1 = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    tuplet_2 = scoretools.FixedDurationTuplet(Duration(2, 8), [])
    tuplet_2.extend("c'8 d'8 e'8 f'8 g'8")
    tuplets = select([tuplet_1, tuplet_2])

    assert pytest.raises(Exception, 'mutate(tuplets).fuse()')


def test_agenttools_MutationAgent_fuse_10():
    r'''Tuplets must be same type.
    '''

    tuplet_1 = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    tuplet_2 = Tuplet(Multiplier(2, 3), "c'8 d'8 e'8")
    tuplets = select([tuplet_1, tuplet_2])

    assert pytest.raises(Exception, 'mutate(tuplets).fuse()')


def test_agenttools_MutationAgent_fuse_11():
    r'''Dominant spanners on contents are preserved.
    '''

    tuplet_1 = scoretools.FixedDurationTuplet(Duration(1, 12), "c'8")
    tuplet_2 = scoretools.FixedDurationTuplet(Duration(1, 6), "c'4")
    voice = Voice([tuplet_1, tuplet_2, Note("c'4")])
    leaves = select(voice).by_leaf()
    slur = Slur()
    attach(slur, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                c'8 (
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                c'4
            }
            c'4 )
        }
        '''
        )

    tuplets = voice[:2]
    mutate(tuplets).fuse()

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \times 2/3 {
                c'8 (
                c'4
            }
            c'4 )
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_fuse_12():
    r'''Fuses unicorporated measures carrying
    time signatures with power-of-two denominators.
    '''

    measure_1 = Measure((1, 8), "c'16 d'16")
    beam = Beam()
    attach(beam, measure_1[:])
    measure_2 = Measure((2, 16), "c'16 d'16")
    slur = Slur()
    attach(slur, measure_2[:])
    staff = Staff([measure_1, measure_2])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/8
                c'16 [
                d'16 ]
            }
            {
                \time 2/16
                c'16 (
                d'16 )
            }
        }
        '''
        )

    new = mutate(staff[:]).fuse()

    assert new is not measure_1 and new is not measure_2
    assert len(measure_1) == 0
    assert len(measure_2) == 0

    assert format(new) == stringtools.normalize(
        r'''
        {
            \time 2/8
            c'16 [
            d'16 ]
            c'16 (
            d'16 )
        }
        '''
        )

    assert inspect_(new).is_well_formed()


def test_agenttools_MutationAgent_fuse_13():
    r'''Fuses measures carrying time signatures with differing
    power-of-two denominators. Helpers selects minimum of two denominators.
    Beams are OK because they attach to leaves rather than containers.
    '''

    voice = Voice("abj: | 1/8 c'16 d'16 || 2/16 e'16 f'16 |")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 1/8
                c'16 [
                d'16
            }
            {
                \time 2/16
                e'16
                f'16 ]
            }
        }
        '''
        )

    mutate(voice[:]).fuse()

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'16 [
                d'16
                e'16
                f'16 ]
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_fuse_14():
    r'''Fuses measures with differing power-of-two denominators.
    Helpers selects minimum of two denominators.
    Beam attaches to container rather than leaves.
    '''

    voice = Voice("abj: | 1/8 c'16 d'16 || 2/16 e'16 f'16 |")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 1/8
                c'16 [
                d'16 ]
            }
            {
                \time 2/16
                e'16
                f'16
            }
        }
        '''
        )

    mutate(voice[:]).fuse()

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'16 [
                d'16 ]
                e'16
                f'16
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_fuse_15():
    r'''Fuses measures with power-of-two-denominators together with measures
    without power-of-two denominators.
    Helpers selects least common multiple of denominators.
    Beams are OK because they attach to leaves rather than containers.
    '''

    measure_1 = Measure((1, 8), "c'8")
    measure_1.implicit_scaling = True
    measure_2 = Measure((1, 12), "d'8")
    measure_2.implicit_scaling = True
    voice = Voice([measure_1, measure_2])
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 1/8
                c'8 [
            }
            {
                \time 1/12
                \scaleDurations #'(2 . 3) {
                    d'8 ]
                }
            }
        }
        '''
        )

    mutate(voice[:]).fuse()

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 5/24
                \scaleDurations #'(2 . 3) {
                    c'8. [
                    d'8 ]
                }
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_fuse_16():
    r'''Fusing empty selection returns none.
    '''

    staff = Staff()
    result = mutate(staff[:]).fuse()
    assert result == selectiontools.Selection()


def test_agenttools_MutationAgent_fuse_17():
    r'''Fusing selection of only one measure returns measure unaltered.
    '''

    measure = Measure((3, 8), "c'8 d'8 e'8")
    staff = Staff([measure])
    new = mutate(staff[:]).fuse()

    assert new is measure


def test_agenttools_MutationAgent_fuse_18():
    r'''Fuses three measures.
    '''

    voice = Voice("abj: | 1/8 c'16 d'16 || 1/8 e'16 f'16 || 1/8 g'16 a'16 |")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 1/8
                c'16 [
                d'16
            }
            {
                e'16
                f'16
            }
            {
                g'16
                a'16 ]
            }
        }
        '''
        )

    mutate(voice[:]).fuse()

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 3/8
                c'16 [
                d'16
                e'16
                f'16
                g'16
                a'16 ]
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_fuse_19():
    r'''Fusing measures with power-of-two denominators
    to measures without power-of-two denominators.
    With change in number of note-heads because of non-power-of-two multiplier.
    '''

    measure_1 = Measure((9, 80), [])
    measure_1.implicit_scaling = True
    measure_2 = Measure((2, 16), [])
    measure_2.implicit_scaling = True
    staff = Staff([measure_1, measure_2])
    scoretools.fill_measures_with_time_signature_denominator_notes(
        staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 9/80
                \scaleDurations #'(4 . 5) {
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                }
            }
            {
                \time 2/16
                c'16
                c'16
            }
        }
        '''
        )

    new = mutate(staff[:]).fuse()

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 19/80
                \scaleDurations #'(4 . 5) {
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'64
                    c'16 ~
                    c'64
                    c'16 ~
                    c'64
                }
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_BendAfter_format_01():
    r'''Bend formats correctly on notes.
    '''

    note = Note("cs'4")
    bend = indicatortools.BendAfter(8)
    attach(bend, note)
    assert format(note) == "cs'4 - \\bendAfter #'8.0"
    detach(indicatortools.BendAfter, note)
    assert format(note) == "cs'4"


def test_indicatortools_BendAfter_format_02():
    r'''Bend formats correctly on chords.
    '''

    chord = Chord([1, 2, 3], (1, 4))
    bend = indicatortools.BendAfter(8)
    attach(bend, chord)
    assert format(chord) == "<cs' d' ef'>4 - \\bendAfter #'8.0"
    detach(indicatortools.BendAfter, chord)
    assert format(chord) == "<cs' d' ef'>4"


def test_indicatortools_BendAfter_format_03():
    r'''Bend formats correctly on rests.
    '''

    rest = Rest((1, 4))
    bend = indicatortools.BendAfter(8)
    attach(bend, rest)
    assert format(rest) == "r4 - \\bendAfter #'8.0"
    detach(indicatortools.BendAfter, rest)
    assert format(rest) == "r4"



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_OctavationSpanner___eq___01():
    r'''Spanner is strict comparator.
    '''

    spanner_1 = spannertools.OctavationSpanner()
    spanner_2 = spannertools.OctavationSpanner()

    assert not spanner_1 == spanner_2



# -*- coding: utf-8 -*-
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools.abctools import AbjadValueObject


class Talea(AbjadValueObject):
    '''Talea.

    ..  container:: example

        ::

            >>> talea = rhythmmakertools.Talea(
            ...    counts=[2, 1, 3, 2, 4, 1, 1],
            ...    denominator=16,
            ...    )

    The medieval plural of 'talea' is 'talee'. Abjad documentation
    uses 'taleas' instead.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Specifiers'

    __slots__ = (
        '_counts',
        '_denominator',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        counts=(1,),
        denominator=16,
        ):
        counts = tuple(counts)
        assert all(isinstance(x, int) for x in counts)
        self._counts = counts
        if not mathtools.is_nonnegative_integer_power_of_two(denominator):
            message = 'denominator must be integer power of 2: {!r}.'
            message = message.format(denominator)
            raise Exception(message)
        self._denominator = denominator

    ### SPECIAL METHODS ###

    def __getitem__(self, argument):
        r'''Gets item or slice identified by `argument`.

        ..  container:: example

            Gets item at index:

            ::

                >>> talea = rhythmmakertools.Talea(
                ...    counts=[2, 1, 3, 2, 4, 1, 1],
                ...    denominator=16,
                ...    )

            ::

                >>> talea[2]
                NonreducedFraction(3, 16)

        ..  container:: example

            Gets items in slice:

            ::

                >>> for nonreduced_fraction in talea[3:9]:
                ...     nonreduced_fraction
                ...
                NonreducedFraction(2, 16)
                NonreducedFraction(4, 16)
                NonreducedFraction(1, 16)
                NonreducedFraction(1, 16)
                NonreducedFraction(2, 16)
                NonreducedFraction(1, 16)

        Returns nonreduced fraction or nonreduced fractions.
        '''
        counts = datastructuretools.CyclicTuple(self.counts)
        if isinstance(argument, int):
            count = counts.__getitem__(argument)
            return mathtools.NonreducedFraction(count, self.denominator)
        elif isinstance(argument, slice):
            counts = counts.__getitem__(argument)
            result = [
                mathtools.NonreducedFraction(count, self.denominator)
                for count in counts
                ]
            return result
        raise ValueError(argument)

    def __iter__(self):
        r'''Iterates talea.

        ..  container:: example

            ::

                >>> talea = rhythmmakertools.Talea(
                ...    counts=[2, 1, 3, 2, 4, 1, 1],
                ...    denominator=16,
                ...    )

            ::

                >>> for duration in talea:
                ...     duration
                ...
                Duration(1, 8)
                Duration(1, 16)
                Duration(3, 16)
                Duration(1, 8)
                Duration(1, 4)
                Duration(1, 16)
                Duration(1, 16)

        Yields durations.
        '''
        for count in self.counts:
            duration = durationtools.Duration(count, self.denominator)
            yield duration

    def __len__(self):
        r'''Gets length.

        ..  container:: example

            ::

                >>> talea = rhythmmakertools.Talea(
                ...    counts=[2, 1, 3, 2, 4, 1, 1],
                ...    denominator=16,
                ...    )

            ::

                >>> len(talea)
                7

        Defined equal to length of counts.

        Returns nonnegative integer.
        '''
        return len(self.counts)

    ### PUBLIC PROPERTIES ###

    @property
    def counts(self):
        r'''Gets counts.

        ..  container:: example

            ::

                >>> talea = rhythmmakertools.Talea(
                ...    counts=[2, 1, 3, 2, 4, 1, 1],
                ...    denominator=16,
                ...    )

            ::

                >>> talea.counts
                [2, 1, 3, 2, 4, 1, 1]

        Set to integers.

        Defaults to `[1]`.

        Returns tuple.
        '''
        if self._counts:
            return list(self._counts)

    @property
    def denominator(self):
        r'''Gets denominator.

        ..  container:: example

            ::

                >>> talea = rhythmmakertools.Talea(
                ...    counts=[2, 1, 3, 2, 4, 1, 1],
                ...    denominator=16,
                ...    )

            ::

                >>> talea.denominator
                16

        Set to nonnegative integer power of two.

        Defaults to 16.

        Returns nonnegative integer power of two.
        '''
        return self._denominator



# -*- coding: utf-8 -*-
import collections
from abjad.tools import mathtools
from abjad.tools.abctools import AbjadValueObject
from abjad.tools.topleveltools.new import new


class Pattern(AbjadValueObject):
    r'''Pattern.

    ..  container:: example

        Matches three indices out of every eight:

        ::

            >>> pattern = patterntools.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=8,
            ...     )

        ::

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2
            3
            4
            5
            6
            7 True
            8 True
            9 True
            10
            11
            12
            13
            14
            15 True

    ..  container:: example

        Matches three indices out of every sixteen:

        ::

            >>> pattern = patterntools.Pattern(
            ...     indices=[0, 1, 7],
            ...     period=16,
            ...     )

        ::

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2
            3
            4
            5
            6
            7 True
            8
            9
            10
            11
            12
            13
            14
            15

    ..  container:: example

        Works with improper indices:

        ::

            >>> pattern = patterntools.Pattern(
            ...     indices=[16, 17, 23],
            ...     period=16,
            ...     )

        ::

            >>> total_length = 16
            >>> for index in range(16):
            ...     match = pattern.matches_index(index, total_length)
            ...     match = match or ''
            ...     print(index, match)
            0 True
            1 True
            2
            3
            4
            5
            6
            7 True
            8
            9
            10
            11
            12
            13
            14
            15

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_indices',
        '_inverted',
        '_payload',
        '_period',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        indices=None,
        inverted=None,
        payload=None,
        period=None,
        ):
        if indices is not None:
            assert all(isinstance(_, int) for _ in indices), repr(indices)
            indices = tuple(indices)
        self._indices = indices
        if inverted is not None:
            inverted = bool(inverted)
        self._inverted = inverted
        if period is not None:
            assert mathtools.is_positive_integer(period), repr(period)
        self._payload = payload
        self._period = period

    ### SPECIAL METHODS ###

    def __and__(self, other):
        r'''Logical AND of two patterns.

        ..  container:: example

            ::

                >>> pattern_1 = patterntools.select_first(3)
                >>> pattern_2 = patterntools.select_last(3)
                >>> pattern = pattern_1 & pattern_2

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0, 1, 2],
                            ),
                        patterntools.Pattern(
                            indices=[-3, -2, -1],
                            ),
                        ),
                    operator='and',
                    )

        Returns compound pattern.
        '''
        from abjad.tools import patterntools
        return patterntools.CompoundPattern([self, other], operator='and')

    def __invert__(self):
        r'''Inverts pattern.

        ..  container:: example

            ::

                >>> pattern = patterntools.select_first(3)
                >>> print(format(pattern))
                patterntools.Pattern(
                    indices=[0, 1, 2],
                    )

            ::

                >>> pattern = ~pattern
                >>> print(format(pattern))
                patterntools.Pattern(
                    indices=[0, 1, 2],
                    inverted=True,
                    )

            ::

                >>> pattern = ~pattern
                >>> print(format(pattern))
                patterntools.Pattern(
                    indices=[0, 1, 2],
                    inverted=False,
                    )

            Negation defined equal to inversion.

        Returns new pattern.
        '''
        inverted = not self.inverted
        pattern = new(self, inverted=inverted)
        return pattern

    def __len__(self):
        r'''Gets length of pattern.

        ..  container:: example

            Gets length of cyclic pattern:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=8,
                ...     )

            ::

                >>> len(pattern)
                8

            Length of cyclic pattern defined equal to period of the pattern.

        ..  container:: example

            Gets length of acyclic pattern:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 2, 3],
                ...     )

            ::

                >>> len(pattern)
                4

            Length of acyclic pattern defined equal to greatest index in
            pattern, plus 1.

        ..  container:: example

            Gets length of pattern with negative indices:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[-3],
                ...     )

            ::

                >>> len(pattern)
                3

            Length of pattern with negative indices defined equal to absolute
            value of least index.

        Returns nonnegative integer.
        '''
        if self.period is not None:
            return self.period
        if self.indices:
            absolute_indices = []
            for index in self.indices:
                if 0 <= index:
                    absolute_indices.append(index)
                else:
                    index = abs(index) - 1
                    absolute_indices.append(index)
            maximum_index = max(absolute_indices)
            return maximum_index + 1
        return 0

    def __or__(self, other):
        r'''Logical OR of two patterns.

        ..  container:: example

            ::

                >>> pattern_1 = patterntools.select_first(3)
                >>> pattern_2 = patterntools.select_last(3)
                >>> pattern = pattern_1 | pattern_2

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0, 1, 2],
                            ),
                        patterntools.Pattern(
                            indices=[-3, -2, -1],
                            ),
                        ),
                    operator='or',
                    )

        Returns compound pattern.
        '''
        from abjad.tools import patterntools
        return patterntools.CompoundPattern([self, other], operator='or')

    def __xor__(self, other):
        r'''Logical XOR of two patterns.

        ..  container:: example

            ::

                >>> pattern_1 = patterntools.select_first(3)
                >>> pattern_2 = patterntools.select_last(3)
                >>> pattern = pattern_1 ^ pattern_2

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0, 1, 2],
                            ),
                        patterntools.Pattern(
                            indices=[-3, -2, -1],
                            ),
                        ),
                    operator='xor',
                    )

        Returns compound pattern.
        '''
        from abjad.tools import patterntools
        return patterntools.CompoundPattern([self, other], operator='xor')

    ### PUBLIC METHODS ###

    @classmethod
    def from_vector(class_, vector):
        r'''Makes pattern from boolean `vector`.

        ..  container:: example

            Matches three indices out of every five:

            ::

                >>> pattern = [1, 0, 0, 1, 1]
                >>> pattern = patterntools.Pattern.from_vector(pattern)
                >>> print(format(pattern))
                patterntools.Pattern(
                    indices=[0, 3, 4],
                    period=5,
                    )

            ::

                >>> total_length = 10
                >>> for index in range(10):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1
                2
                3 True
                4 True
                5 True
                6
                7
                8 True
                9 True

        ..  container:: example

            Matches three indices out of every six:

            ::

                >>> pattern = [1, 0, 0, 1, 1, 0]
                >>> pattern = patterntools.Pattern.from_vector(pattern)
                >>> print(format(pattern))
                patterntools.Pattern(
                    indices=[0, 3, 4],
                    period=6,
                    )

            ::

                >>> total_length = 12
                >>> for index in range(12):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1
                2
                3 True
                4 True
                5
                6 True
                7
                8
                9 True
                10 True
                11

        Returns pattern.
        '''
        vector = [bool(_) for _ in vector]
        period = len(vector)
        indices = [i for i, x in enumerate(vector) if x]
        return class_(
            period=period,
            indices=indices,
            )

    def get_boolean_vector(self, total_length=None):
        r'''Gets boolean vector of pattern applied to input sequence with
        `total_length`.

        ..  container:: example

            Gets boolean vector of acyclic pattern:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[4, 5, 6, 7],
                ...     )


            ::

                >>> pattern.get_boolean_vector(4)
                [0, 0, 0, 0]

            ::

                >>> pattern.get_boolean_vector(8)
                [0, 0, 0, 0, 1, 1, 1, 1]

            ::

                >>> pattern.get_boolean_vector(16)
                [0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0]

            Sets total length to length of pattern when `total_length` is none:

            ::

                >>> pattern.get_boolean_vector()
                [0, 0, 0, 0, 1, 1, 1, 1]

        ..  container:: example

            Gets vector of cyclic pattern:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[4, 5, 6, 7],
                ...     period=20,
                ...     )

            ::

                >>> pattern.get_boolean_vector(4)
                [0, 0, 0, 0]

            ::

                >>> pattern.get_boolean_vector(8)
                [0, 0, 0, 0, 1, 1, 1, 1]

            ::

                >>> pattern.get_boolean_vector(16)
                [0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0]

            Sets total length to length of pattern when `total_length` is none:

            ::

                >>> pattern.get_boolean_vector()
                [0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]

        ..  container:: example

            Gets vector of inverted pattern:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[4, 5, 6, 7],
                ...     period=20,
                ...     )

            ::

                >>> pattern.get_boolean_vector(4)
                [0, 0, 0, 0]

            ::

                >>> pattern.get_boolean_vector(8)
                [0, 0, 0, 0, 1, 1, 1, 1]

            ::

                >>> pattern.get_boolean_vector(16)
                [0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0]

            Sets total length to length of pattern when `total_length` is none:

            ::

                >>> pattern.get_boolean_vector()
                [0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]

        Returns list of ones and zeroes.
        '''
        total_length = total_length or len(self)
        boolean_vector = []
        for index in range(total_length):
            result = self.matches_index(index, total_length)
            boolean_vector.append(int(result))
        return boolean_vector

    def get_matching_items(self, sequence):
        r'''Gets maching items from sequence.

        ..  container:: example

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[4, 5, 6, 7],
                ...     )

            ::

                >>> pattern.get_matching_items('abcdefghijklmnopqrstuvwxyz')
                Sequence(['e', 'f', 'g', 'h'])

        ..  container:: example

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[8, 9],
                ...     period=10,
                ...     )

            ::

                >>> pattern.get_matching_items('abcdefghijklmnopqrstuvwxyz')
                Sequence(['i', 'j', 's', 't'])

        Returns new sequence.
        '''
        import abjad
        assert isinstance(sequence, collections.Iterable), repr(sequence)
        length = len(sequence)
        items = []
        for i in range(length):
            if self.matches_index(i, length):
                item = sequence[i]
                items.append(item)
        return abjad.Sequence(items=items)

    def matches_index(self, index, total_length, rotation=None):
        r'''Is true when pattern matches `index` taken under `total_length`.
        Otherwise false.

        ..  container:: example

            Matches three indices out of every eight:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=8,
                ...     )

            ::

                >>> total_length = 16
                >>> for index in range(16):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2
                3
                4
                5
                6
                7 True
                8 True
                9 True
                10
                11
                12
                13
                14
                15 True

            Matches three indices out of every eight, offset ``1`` to the left:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=8,
                ...     )

            ::

                >>> total_length = 16
                >>> for index in range(16):
                ...     match = pattern.matches_index(
                ...         index,
                ...         total_length,
                ...         rotation=1,
                ...         )
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1
                2
                3
                4
                5
                6 True
                7 True
                8 True
                9
                10
                11
                12
                13
                14 True
                15 True

             Matches three indices out of every eight, offset ``2`` to the
             left:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=8,
                ...     )

            ::

                >>> total_length = 16
                >>> for index in range(16):
                ...     match = pattern.matches_index(
                ...         index,
                ...         total_length,
                ...         rotation=2,
                ...         )
                ...     match = match or ''
                ...     print(index, match)
                0
                1
                2
                3
                4
                5 True
                6 True
                7 True
                8
                9
                10
                11
                12
                13 True
                14 True
                15 True

        ..  container:: example

            Matches three indices out of every sixteen:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=16,
                ...     )

            ::

                >>> total_length = 16
                >>> for index in range(16):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2
                3
                4
                5
                6
                7 True
                8
                9
                10
                11
                12
                13
                14
                15

            Matches three indices out of every sixteen, offset ``1`` to the
            left:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=16,
                ...     )

            ::

                >>> total_length = 16
                >>> for index in range(16):
                ...     match = pattern.matches_index(
                ...         index,
                ...         total_length,
                ...         rotation=1,
                ...         )
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1
                2
                3
                4
                5
                6 True
                7
                8
                9
                10
                11
                12
                13
                14
                15 True

            Matches three indices out of every sixteen, offset ``2`` to the
            left:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=16,
                ...     )

            ::

                >>> total_length = 16
                >>> for index in range(16):
                ...     match = pattern.matches_index(
                ...         index,
                ...         total_length,
                ...         rotation=2,
                ...         )
                ...     match = match or ''
                ...     print(index, match)
                0
                1
                2
                3
                4
                5 True
                6
                7
                8
                9
                10
                11
                12
                13
                14 True
                15 True

        Returns true or false.
        '''
        assert 0 <= total_length
        if 0 <= index:
            nonnegative_index = index
        else:
            nonnegative_index = total_length - abs(index)
        inverted = bool(self.inverted)
        if self.period is None:
            for index in self.indices:
                if index < 0:
                    index = total_length - abs(index)
                if index == nonnegative_index and index < total_length:
                    return True ^ inverted
        else:
            if rotation is not None:
                nonnegative_index += rotation
            nonnegative_index = nonnegative_index % self.period
            for index in self.indices:
                if index < 0:
                    index = total_length - abs(index)
                    index = index % self.period
                if index == nonnegative_index and index < total_length:
                    return True ^ inverted
                if ((index % self.period) == nonnegative_index and
                    (index % self.period < total_length)):
                    return True ^ inverted
        return False ^ inverted

    def reverse(self):
        r'''Reverses pattern.

        ..  container:: example

            Matches three indices out of every eight:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=8,
                ...     )

            ::

                >>> total_length = 16
                >>> for index in range(16):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2
                3
                4
                5
                6
                7 True
                8 True
                9 True
                10
                11
                12
                13
                14
                15 True

        ..  container:: example

            Reverses pattern:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=8,
                ...     )

            ::

                >>> pattern = pattern.reverse()
                >>> print(format(pattern))
                patterntools.Pattern(
                    indices=[-1, -2, -8],
                    period=8,
                    )

            ::

                >>> total_length = 16
                >>> for index in range(16):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1
                2
                3
                4
                5
                6 True
                7 True
                8 True
                9
                10
                11
                12
                13
                14 True
                15 True

        Returns new pattern.
        '''
        indices = [-index - 1 for index in self.indices]
        return new(self, indices=indices)

    def rotate(self, n=0):
        r'''Rotates pattern by index `n`.

        ..  container:: example

            Matches three indices out of every eight:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=8,
                ...     )

            ::

                >>> total_length = 16
                >>> for index in range(16):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2
                3
                4
                5
                6
                7 True
                8 True
                9 True
                10
                11
                12
                13
                14
                15 True

            Rotates pattern two elements to the right:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=8,
                ...     )

            ::

                >>> pattern = pattern.rotate(n=2)
                >>> print(format(pattern))
                patterntools.Pattern(
                    indices=[2, 3, 9],
                    period=8,
                    )

            ::

                >>> total_length = 16
                >>> for index in range(16):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0
                1 True
                2 True
                3 True
                4
                5
                6
                7
                8
                9 True
                10 True
                11 True
                12
                13
                14
                15

        ..  container:: example

            Matches three indices out of every eight with negative indices:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[-3, -2, -1],
                ...     period=8,
                ...     )

            ::

                >>> total_length = 16
                >>> for index in range(16):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0
                1
                2
                3
                4
                5 True
                6 True
                7 True
                8
                9
                10
                11
                12
                13 True
                14 True
                15 True

            Rotates pattern two elements to the right:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[-3, -2, -1],
                ...     period=8,
                ...     )

            ::

                >>> pattern = pattern.rotate(n=2)
                >>> print(format(pattern))
                patterntools.Pattern(
                    indices=[-1, 0, 1],
                    period=8,
                    )

            ::

                >>> total_length = 16
                >>> for index in range(16):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2
                3
                4
                5
                6
                7 True
                8 True
                9 True
                10
                11
                12
                13
                14
                15 True

        Returns new pattern.
        '''
        indices = [index + n for index in self.indices]
        return new(self, indices=indices)

    ### PUBLIC PROPERTIES ###

    @property
    def indices(self):
        r'''Gets indices of pattern.

        ..  container:: example

            Matches three indices out of every eight:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=8,
                ...     )

            ::

                >>> pattern.indices
                [0, 1, 7]

        ..  container:: example

            Matches three indices out of every sixteen:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=16,
                ...     )

            ::

                >>> pattern.indices
                [0, 1, 7]

        Defaults to none.

        Set to integers or none.

        Returns integers or none.
        '''
        if self._indices:
            return list(self._indices)
        return []

    @property
    def inverted(self):
        r'''Is true when pattern is inverted. Otherwise false.

        ..  container:: example

            Matches three indices out of every eight:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=8,
                ...     )

            ::

                >>> pattern.inverted is None
                True

            ::

                >>> total_length = 16
                >>> for index in range(16):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2
                3
                4
                5
                6
                7 True
                8 True
                9 True
                10
                11
                12
                13
                14
                15 True

        ..  container:: example

            Pattern that rejects three indices from every eight; equivalently,
            pattern matches ``8-3=5`` indices out of every eight:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=8,
                ...     inverted=True
                ...     )

            ::

                >>> pattern.inverted
                True

            ::

                >>> total_length = 16
                >>> for index in range(16):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0
                1
                2 True
                3 True
                4 True
                5 True
                6 True
                7
                8
                9
                10 True
                11 True
                12 True
                13 True
                14 True
                15

        Defaults to none.

        Set to true, false or none.

        Returns true, false or none.
        '''
        return self._inverted

    @property
    def payload(self):
        r'''Gets payload of pattern.

        ..  container:: example

            Pattern with rhythm-maker payload assigned to three
            of every eight indices:

            ::

                >>> maker = rhythmmakertools.NoteRhythmMaker()
                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     payload=maker,
                ...     period=8,
                ...     )

            ::

                >>> total_length = 10
                >>> for index in range(10):
                ...     match = pattern.matches_index(index, total_length)
                ...     if match:
                ...         payload = pattern.payload
                ...     else:
                ...         payload = ''
                ...     print(index, payload)
                0 NoteRhythmMaker()
                1 NoteRhythmMaker()
                2
                3
                4
                5
                6
                7 NoteRhythmMaker()
                8 NoteRhythmMaker()
                9 NoteRhythmMaker()

        Defaults to none.

        Set to any object.

        Returns arbitrary object.
        '''
        return self._payload

    @property
    def period(self):
        r'''Gets period of pattern.

        ..  container:: example

            Pattern with a period of eight:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=8,
                ...     )

            ::

                >>> pattern.period
                8

            ::

                >>> total_length = 16
                >>> for index in range(16):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2
                3
                4
                5
                6
                7 True
                8 True
                9 True
                10
                11
                12
                13
                14
                15 True

        ..  container:: example

            Same pattern with a period of sixteen:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=16,
                ...     )

            ::

                >>> pattern.period
                16

            ::

                >>> total_length = 16
                >>> for index in range(16):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2
                3
                4
                5
                6
                7 True
                8
                9
                10
                11
                12
                13
                14
                15

        Defaults to none.

        Set to positive integer or none.

        Returns positive integer or none.
        '''
        return self._period

    @property
    def weight(self):
        r'''Gets weight of pattern.

        ..  container:: example

            Gets weight of cyclic pattern:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 1, 7],
                ...     period=8,
                ...     )

            ::

                >>> pattern.weight
                3

        ..  container:: example

            Gets weight of acyclic pattern:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0, 2, 3],
                ...     )

            ::

                >>> pattern.weight
                3

        Weight defined equal to number of indices in pattern.

        Returns nonnegative integer.
        '''
        return len(self.indices)



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_TextSpanner_position_01():

    staff = Staff("c'8 c'8 c'8 c'8")
    text_spanner = spannertools.TextSpanner()
    attach(text_spanner, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 \startTextSpan
            c'8
            c'8
            c'8 \stopTextSpan
        }
        '''
        )


def test_spannertools_TextSpanner_position_02():

    staff = Staff("c'8 c'8 c'8 c'8")
    text_spanner = spannertools.TextSpanner()
    attach(text_spanner, staff[:])
    command = indicatortools.LilyPondCommand('textSpannerNeutral')
    attach(command, text_spanner[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \textSpannerNeutral
            c'8 \startTextSpan
            c'8
            c'8
            c'8 \stopTextSpan
        }
        '''
        )


def test_spannertools_TextSpanner_position_03():

    staff = Staff("c'8 c'8 c'8 c'8")
    text_spanner = spannertools.TextSpanner()
    attach(text_spanner, staff[:])
    command = indicatortools.LilyPondCommand('textSpannerUp')
    attach(command, text_spanner[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \textSpannerUp
            c'8 \startTextSpan
            c'8
            c'8
            c'8 \stopTextSpan
        }
        '''
        )


def test_spannertools_TextSpanner_position_04():

    staff = Staff("c'8 c'8 c'8 c'8")
    text_spanner = spannertools.TextSpanner()
    attach(text_spanner, staff[:])
    command = indicatortools.LilyPondCommand('textSpannerDown')
    attach(command, text_spanner[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \textSpannerDown
            c'8 \startTextSpan
            c'8
            c'8
            c'8 \stopTextSpan
        }
        '''
        )


def test_spannertools_TextSpanner_position_05():
    r'''TextSpanner attaching to container formats correctly.
    '''

    container = Container("c'8 c'8 c'8 c'8")
    text_spanner = spannertools.TextSpanner()
    attach(text_spanner, container[:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8 \startTextSpan
            c'8
            c'8
            c'8 \stopTextSpan
        }
        '''
        )


# -*- coding: utf-8 -*-
import math
import numbers
import re
from abjad.tools import systemtools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class Octave(AbjadValueObject):
    r'''Octave.

    ..  container:: example:

        Initializes octave from integer:

        ::

            >>> pitchtools.Octave(4)
            Octave(4)

    ..  container:: example

        Initializes octave from octave-tick string:

        ::

            >>> pitchtools.Octave(",,")
            Octave(1)

    ..  container:: example

        Initializes octave from named pitch:

        ::

            >>> pitchtools.Octave(NamedPitch("cs''"))
            Octave(5)

    ..  container:: example

        Initializes octave from other octave:

        ::

            >>> pitchtools.Octave(pitchtools.Octave(2))
            Octave(2)

    '''

    ### CLASS VARIABLES ###

    _octave_tick_regex_body = """
        (,+     # one or more commas for octaves below the bass clef
        |'+     # or one or more apostrophes for the octave of the treble clef
        |)      # or empty string for the octave of the bass clef
        """

    _octave_tick_regex = re.compile(
        '^{}$'.format(_octave_tick_regex_body),
        re.VERBOSE,
        )

    __slots__ = (
        '_number',
        )

    ### INITIALIZER ###

    def __init__(self, number=None):
        from abjad.tools import pitchtools
        argument = number
        if isinstance(argument, numbers.Number):
            number = int(argument)
        elif isinstance(argument, str):
            match = self._octave_tick_regex.match(argument)
            if match is None:
                message = 'can not instantiate octave: {!r}.'
                message = message.format(argument)
                raise Exception(message)
            group = match.group()
            if group == '':
                number = 3
            elif group.startswith("'"):
                number = 3 + len(group)
            else:
                number = 3 - len(group)
        elif isinstance(argument, pitchtools.Pitch):
            number = argument.octave.number
        elif isinstance(argument, type(self)):
            number = argument.number
        elif argument is None:
            number = 4
        else:
            message = 'can not instantiate {}: {!r}.'
            message = message.format(type(self), argument)
            raise Exception(message)
        self._number = number

    ### SPECIAL METHODS ###

    def __eq__(self, other):
        r'''Is true when `other` is octave with same octave number.
        Otherwise False.

        ..  container:: example

            ::

                >>> octave = pitchtools.Octave(4)
                >>> octave == pitchtools.Octave(4)
                True

            ::

                >>> octave == pitchtools.Octave(3)
                False

            ::

                >>> octave == 'foo'
                False

        Returns true or false.
        '''
        try:
            other = type(self)(other)
            return self.number == other.number
        except:
            return False

    def __float__(self):
        r'''Casts octave as floating-point number.

        ..  container:: example

            ::

                >>> float(pitchtools.Octave(3))
                3.0

        Returns floating-point number.
        '''
        return float(self.number)

    def __hash__(self):
        r'''Hashes octave.

        Returns integer.
        '''
        return hash(repr(self))

    def __int__(self):
        r'''Changes octave to integer.

        ..  container:: example

            ::

                >>> int(pitchtools.Octave(3))
                3

        Returns integer.
        '''
        return self.number

    def __str__(self):
        r'''Gets string representation of octave.

        Defined equal to LilyPond octave / tick representation of octave.

        ..  container:: example

            ::

                >>> str(pitchtools.Octave(4))
                "'"

            ::

                >>> str(pitchtools.Octave(1))
                ',,'

            ::

                >>> str(pitchtools.Octave(3))
                ''

        Returns string.
        '''
        if 3 < self.number:
            return "'" * (self.number - 3)
        elif self.number < 3:
            return ',' * abs(3 - self.number)
        return ''

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=False,
            storage_format_is_indented=False,
            storage_format_args_values=[self.number],
            storage_format_kwargs_names=[],
            )

    ### PUBLIC PROPERTIES ###

    @property
    def number(self):
        r'''Gets octave number.

        ..  container:: example

            ::

                >>> pitchtools.Octave(5).number
                5

        Returns integer.
        '''
        return self._number

    @property
    def pitch_number(self):
        r'''Gets pitch number of first note in octave.

        ..  container:: example

            ::

                >>> pitchtools.Octave(4).pitch_number
                0

            ::

                >>> pitchtools.Octave(5).pitch_number
                12

            ::

                >>> pitchtools.Octave(3).pitch_number
                -12

        Returns integer.
        '''
        return (self.number - 4) * 12

    @property
    def pitch_range(self):
        r'''Gets pitch range of octave.

        ..  container:: example

            ::

                >>> pitchtools.Octave(5).pitch_range
                PitchRange(range_string='[C5, C6)')

        Returns pitch range.
        '''
        from abjad.tools import pitchtools
        return pitchtools.PitchRange(
            '[C{}, C{})'.format(
                self.number,
                self.number + 1,
                ))

    @property
    def tick_string(self):
        r"""Gets LilyPond octave tick string.

        ..  container:: example

            ::

                >>> for i in range(-1, 9):
                ...     print(i, pitchtools.Octave(i).tick_string)
                -1 ,,,,
                0  ,,,
                1  ,,
                2  ,
                3
                4  '
                5  ''
                6  '''
                7  ''''
                8  '''''

        Returns string.
        """
        return str(self)

    ### PUBLIC METHODS ###

    @classmethod
    def from_pitch_name(class_, pitch_name):
        '''Makes octave from `pitch_name`.

        ..  container:: example

            ::

                >>> pitchtools.Octave.from_pitch_name('cs')
                Octave(3)

        Returns integer.
        '''
        if not isinstance(pitch_name, str):
            message = 'must be string: {!r}.'
            message = message.format(pitch_name)
            raise TypeError(message)
        match = re.match('^([a-z]+)(\,*|\'*)$', pitch_name)
        if match is None:
            message = 'incorrect pitch string format.'
            raise TypeError(message)
        name, tick_string = match.groups()
        return class_(tick_string)

    @classmethod
    def from_pitch_number(class_, pitch_number):
        r'''Makes octave from `pitch_number`.

        ..  container:: example

            ::

                >>> pitchtools.Octave.from_pitch_number(13)
                Octave(5)

        Returns octave.
        '''
        number = int(math.floor(pitch_number / 12)) + 4
        return class_(number)

    @classmethod
    def is_tick_string(class_, argument):
        r"""Is true when `argument` is an octave tick string.
        Otherwise false.

        ..  container:: example

            ::

                >>> pitchtools.Octave.is_tick_string(',,,')
                True

            ::

                >>> pitchtools.Octave.is_tick_string(',,,')
                True

            ::
            
                >>> pitchtools.Octave.is_tick_string(',,')
                True

            ::

                >>> pitchtools.Octave.is_tick_string(',')
                True

            ::

                >>> pitchtools.Octave.is_tick_string('')
                True

            ::

                >>> pitchtools.Octave.is_tick_string("")
                True

            ::

                >>> pitchtools.Octave.is_tick_string("'")
                True

            ::

                >>> pitchtools.Octave.is_tick_string("''")
                True

            ::

                >>> pitchtools.Octave.is_tick_string("'''")
                True

            ::

                >>> pitchtools.Octave.is_tick_string('foo')
                False

            ::

                >>> pitchtools.Octave.is_tick_string(8)
                False

        The regex ``^,+|'+|$`` underlies this predicate.

        Returns true or false.
        """
        if not isinstance(argument, str):
            return False
        return bool(class_._octave_tick_regex.match(argument))



# -*- coding: utf-8 -*-
from abjad.tools.datastructuretools import TreeNode


class GraphvizTableCell(TreeNode):
    r'''A Graphviz table cell.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Graphviz'

    __slots__ = (
        '_attributes',
        '_label',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        label=None,
        name=None,
        attributes=None,
        ):
        TreeNode.__init__(
            self,
            name=name,
            )
        self._label = label
        if attributes is not None:
            attributes = dict(attributes)
        self._attributes = attributes

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Get string representation of Graphviz table cell.

        Returns string.
        '''
        result = []
        attribute_string = self._attribute_string
        if attribute_string:
            result.append('<TD {}>'.format(attribute_string))
        else:
            result.append('<TD>')
        result.append(self.label or '')
        result.append('</TD>')
        result = ''.join(result)
        return result

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_string(self):
        if not self.attributes:
            return ''
        result = []
        for key, value in sorted(self.attributes.items()):
            attribute = '{}="{}"'.format(key.upper(), str(value).upper())
            result.append(attribute)
        result = ' '.join(result)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def attributes(self):
        r'''Gets Graphviz table cell attribute dictionary.

        Returns dictionary.
        '''
        return self._attributes

    @property
    def label(self):
        r'''Gets the field label.
        '''
        return self._label



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Container__get_spanners_that_dominate_slice_01():
    r'''Get dominant spanners over zero-length slice.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:2])
    glissando = spannertools.Glissando()
    attach(glissando, voice[:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \glissando
            d'8 ] \glissando
            e'8 \glissando
            f'8
        }
        '''
        )

    receipt = voice._get_spanners_that_dominate_slice(2, 2)

    assert len(receipt) == 1
    assert (glissando, 2) in receipt


def test_scoretools_Container__get_spanners_that_dominate_slice_02():
    r'''Get dominant spanners over one-component slice.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:2])
    glissando = spannertools.Glissando()
    attach(glissando, voice[:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \glissando
            d'8 ] \glissando
            e'8 \glissando
            f'8
        }
        '''
        )

    receipt = voice._get_spanners_that_dominate_slice(1, 2)

    assert len(receipt) == 2
    assert (beam, 1) in receipt
    assert (glissando, 1) in receipt


def test_scoretools_Container__get_spanners_that_dominate_slice_03():
    r'''Get dominant spanners over four-component slice.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:2])
    glissando = spannertools.Glissando()
    attach(glissando, voice[:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \glissando
            d'8 ] \glissando
            e'8 \glissando
            f'8
        }
        '''
        )

    receipt = voice._get_spanners_that_dominate_slice(0, 4)

    assert len(receipt) == 1
    assert (glissando, 0) in receipt



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_UnweightedSearchTree___init___01():

    search_tree = quantizationtools.UnweightedSearchTree()
    assert search_tree.definition == search_tree.default_definition


def test_quantizationtools_UnweightedSearchTree___init___02():

    definition = {2: None, 3: {2: None}}
    search_tree = quantizationtools.UnweightedSearchTree(definition)
    assert search_tree.definition == definition



# -*- coding: utf-8 -*-
import copy
from abjad import *


def test_lilypondproxytools_LilyPondNameManager___eq___01():

    note_1 = Note("c'4")
    set_(note_1).voice.auto_beaming = False
    set_(note_1).voice.tuplet_full_length = True

    note_2 = Note("c'4")
    set_(note_2).voice.auto_beaming = False
    set_(note_2).voice.tuplet_full_length = True

    note_3 = Note("c'4")
    set_(note_3).voice.auto_beaming = True

    context_proxy_1 = set_(note_1).voice
    context_proxy_2 = set_(note_2).voice
    context_proxy_3 = set_(note_3).voice

    assert      context_proxy_1 == context_proxy_1
    assert      context_proxy_1 == context_proxy_2
    assert not context_proxy_1 == context_proxy_3
    assert      context_proxy_2 == context_proxy_1
    assert      context_proxy_2 == context_proxy_2
    assert not context_proxy_2 == context_proxy_3
    assert not context_proxy_3 == context_proxy_1
    assert not context_proxy_3 == context_proxy_2
    assert      context_proxy_3 == context_proxy_3


def test_lilypondproxytools_LilyPondNameManager___eq___02():

    note_1 = Note("c'4")
    override(note_1).note_head.color = 'red'
    override(note_1).note_head.thickness = 2

    note_2 = Note("c'4")
    override(note_2).note_head.color = 'red'
    override(note_2).note_head.thickness = 2

    note_3 = Note("c'4")
    override(note_3).note_head.color = 'blue'

    grob_proxy_1 = override(note_1).note_head
    grob_proxy_2 = override(note_2).note_head
    grob_proxy_3 = override(note_3).note_head

    assert      grob_proxy_1 == grob_proxy_1
    assert      grob_proxy_1 == grob_proxy_2
    assert not grob_proxy_1 == grob_proxy_3
    assert      grob_proxy_2 == grob_proxy_1
    assert      grob_proxy_2 == grob_proxy_2
    assert not grob_proxy_2 == grob_proxy_3
    assert not grob_proxy_3 == grob_proxy_1
    assert not grob_proxy_3 == grob_proxy_2
    assert      grob_proxy_3 == grob_proxy_3



# -*- coding: utf-8 -*-
import platform
import pytest
import sys
from abjad import *
from abjad.tools import systemtools


@pytest.mark.skipif(
    platform.python_implementation() != 'CPython',
    reason='Benchmarking is only for CPython.',
    )
def test_mathtools_NonreducedFraction___init___01():
    if sys.version_info[0] == 2:
        result = systemtools.IOManager.count_function_calls(
            'Fraction(3, 6)', globals())
        assert result in (2, 13)
        result = systemtools.IOManager.count_function_calls(
            'mathtools.NonreducedFraction(3, 6)', globals())
        assert result in (20, 30)
    else:
        result = systemtools.IOManager.count_function_calls(
            'Fraction(3, 6)', globals())
        assert result < 20
        result = systemtools.IOManager.count_function_calls(
            'mathtools.NonreducedFraction(3, 6)', globals())
        assert result < 100



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.quantizationtools import *


def test_quantizationtools_QGrid_distance_01():

    q_grid = QGrid()

    assert q_grid.distance is None

    a = QEventProxy(SilentQEvent(0,        ['A']), 0)
    q_grid.fit_q_events([a])
    assert q_grid.distance == Offset(0)

    b = QEventProxy(SilentQEvent((1, 20),  ['B']), (1, 20))
    q_grid.fit_q_events([b])
    assert q_grid.distance == Offset(1, 40)

    c = QEventProxy(SilentQEvent((9, 20),  ['C']), (9, 20))
    q_grid.fit_q_events([c])
    assert q_grid.distance == Offset(1, 6)

    d = QEventProxy(SilentQEvent((1, 2),   ['D']), (1, 2))
    q_grid.fit_q_events([d])
    assert q_grid.distance == Offset(1, 4)

    e = QEventProxy(SilentQEvent((11, 20), ['E']), (11, 20))
    q_grid.fit_q_events([e])
    assert q_grid.distance == Offset(29, 100)

    f = QEventProxy(SilentQEvent((19, 20), ['F']), (19, 20))
    q_grid.fit_q_events([f])
    assert q_grid.distance == Offset(1, 4)

    g = QEventProxy(SilentQEvent(1,        ['G']), 1)
    q_grid.fit_q_events([g])
    assert q_grid.distance == Offset(3, 14)

    q_events = q_grid.subdivide_leaves([(0, (1, 1))])
    q_grid.fit_q_events(q_events)

    assert q_grid.distance == Offset(1, 35)



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadValueObject
from abjad.tools import mathtools
from abjad.tools import selectiontools
from abjad.tools import sequencetools


class PartitionByRatioCallback(AbjadValueObject):
    r'''Partition-by-ratio selector callback.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Callbacks'

    __slots__ = (
        '_ratio',
        )

    ### INITIALIZER ###

    def __init__(self, ratio=None):
        ratio = ratio or mathtools.Ratio((1,))
        ratio = mathtools.Ratio(ratio)
        self._ratio = ratio

    ### SPECIAL METHODS ###

    def __call__(self, argument, rotation=None):
        r'''Calls ratio selector callback on `argument`.

        Returns tuple of selections.
        '''
        assert isinstance(argument, tuple), repr(argument)
        assert len(argument) == 1, repr(argument)
        assert isinstance(argument[0], selectiontools.Selection), repr(argument)
        selection = argument[0]
        counts = mathtools.partition_integer_by_ratio(
            len(selection),
            self.ratio,
            )
        selections = sequencetools.Sequence(selection).partition_by_counts(
            counts=counts,
            )
        selections = [selectiontools.Selection(_) for _ in selections]
        return tuple(selections)

    ### PUBLIC PROPERTIES ###

    @property
    def ratio(self):
        r'''Gets ratio.

        Returns ratio.
        '''
        return self._ratio



# -*- coding: utf-8 -*-
import os
import platform
import json
import shutil
from abjad.tools import systemtools
from base import ScorePackageScriptTestCase


class Test(ScorePackageScriptTestCase):

    expected_files = [
        'test_score/.gitignore',
        'test_score/.travis.yml',
        'test_score/README.md',
        'test_score/requirements.txt',
        'test_score/setup.cfg',
        'test_score/setup.py',
        'test_score/test_score/__init__.py',
        'test_score/test_score/__metadata__.py',
        'test_score/test_score/build/.gitignore',
        'test_score/test_score/build/assets/.gitignore',
        'test_score/test_score/build/assets/instrumentation.tex',
        'test_score/test_score/build/assets/performance-notes.tex',
        'test_score/test_score/build/parts.ily',
        'test_score/test_score/build/segments.ily',
        'test_score/test_score/build/segments/.gitignore',
        'test_score/test_score/distribution/.gitignore',
        'test_score/test_score/etc/.gitignore',
        'test_score/test_score/materials/.gitignore',
        'test_score/test_score/materials/__init__.py',
        'test_score/test_score/metadata.json',
        'test_score/test_score/segments/.gitignore',
        'test_score/test_score/segments/__init__.py',
        'test_score/test_score/stylesheets/.gitignore',
        'test_score/test_score/stylesheets/nonfirst-segment.ily',
        'test_score/test_score/stylesheets/parts.ily',
        'test_score/test_score/stylesheets/stylesheet.ily',
        'test_score/test_score/test/.gitignore',
        'test_score/test_score/test/test_materials.py',
        'test_score/test_score/test/test_segments.py',
        'test_score/test_score/tools/.gitignore',
        'test_score/test_score/tools/ScoreTemplate.py',
        'test_score/test_score/tools/SegmentMaker.py',
        'test_score/test_score/tools/__init__.py',
        ]

    if platform.system().lower() == 'windows':
        expected_files = [_.replace('/', os.path.sep) for _ in expected_files]

    def test_exists(self):
        with systemtools.RedirectedStreams(stdout=self.string_io):
            self.create_score()
        assert self.score_path.exists()
        with systemtools.RedirectedStreams(stdout=self.string_io):
            self.create_score(expect_error=True)
        assert self.score_path.exists()
        shutil.rmtree(str(self.score_path))
        for path in self.test_path.iterdir():
            assert path in self.directory_items
        self.compare_captured_output(r'''
            Creating score package 'Test Score'...
                Writing test_score/metadata.json
                Created test_score/
            Creating score package 'Test Score'...
                Directory test_score already exists.
        '''.replace('/', os.path.sep))

    def test_force_replace(self):
        with systemtools.RedirectedStreams(stdout=self.string_io):
            self.create_score()
        assert self.score_path.exists()
        with systemtools.RedirectedStreams(stdout=self.string_io):
            self.create_score(force=True)
        assert self.score_path.exists()
        shutil.rmtree(str(self.score_path))
        for path in self.test_path.iterdir():
            assert path in self.directory_items
        self.compare_captured_output(r'''
            Creating score package 'Test Score'...
                Writing test_score/metadata.json
                Created test_score/
            Creating score package 'Test Score'...
                Writing test_score/metadata.json
                Created test_score/
        '''.replace('/', os.path.sep))

    def test_success(self):
        with systemtools.RedirectedStreams(stdout=self.string_io):
            self.create_score()
        assert self.score_path.exists()
        self.compare_path_contents(self.score_path, self.expected_files)
        score_metadata_path = self.score_path.joinpath(
            self.score_path.name, 'metadata.json')
        assert score_metadata_path.exists()
        with open(str(score_metadata_path), 'r') as file_pointer:
            metadata = json.loads(file_pointer.read())
        assert metadata == {
            'composer_email': 'josiah.oberholtzer@gmail.com',
            'composer_github': 'josiah-wolf-oberholtzer',
            'composer_library': 'consort',
            'composer_name': 'Josiah Wolf Oberholtzer',
            'composer_website': 'www.josiahwolfoberholtzer.com',
            'title': 'Test Score',
            'year': 2016,
            }
        shutil.rmtree(str(self.score_path))
        for path in self.test_path.iterdir():
            assert path in self.directory_items
        self.compare_captured_output(r'''
            Creating score package 'Test Score'...
                Writing test_score/metadata.json
                Created test_score/
        '''.replace('/', os.path.sep))



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_ComplexBeam_direction_01():

    staff = Staff("c'16 e'16 r16 f'16 g'2")
    beam = spannertools.ComplexBeam(direction=Up)
    attach(beam, staff[:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set stemLeftBeamCount = #0
            \set stemRightBeamCount = #2
            c'16 ^ [
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #2
            e'16 ]
            r16
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #0
            f'16 ^ [ ]
            g'2
        }
        '''
        )


def test_spannertools_ComplexBeam_direction_02():

    staff = Staff("c'16 e'16 r16 f'16 g'2")
    beam = spannertools.ComplexBeam(direction=Down)
    attach(beam, staff[:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set stemLeftBeamCount = #0
            \set stemRightBeamCount = #2
            c'16 _ [
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #2
            e'16 ]
            r16
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #0
            f'16 _ [ ]
            g'2
        }
        '''
        )



# -*- coding: utf-8 -*-
import pytest
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__spanners__Hairpin_01():

    target = Staff(scoretools.make_notes([0] * 5, [(1, 4)]))
    hairpin = Hairpin(descriptor='<')
    attach(hairpin, target[:3])
    hairpin = Hairpin(descriptor='>')
    attach(hairpin, target[2:])
    dynamic = Dynamic('ppp')
    attach(dynamic, target[-1])

    assert format(target) == stringtools.normalize(
        r'''
        \new Staff {
            c'4 \<
            c'4
            c'4 \! \>
            c'4
            c'4 \ppp
        }
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__spanners__Hairpin_02():

    target = Container(scoretools.make_notes([0] * 4, [(1, 4)]))
    hairpin = Hairpin(descriptor='<')
    attach(hairpin, target[0:2])
    hairpin = Hairpin(descriptor='<')
    attach(hairpin, target[1:3])
    hairpin = Hairpin(descriptor='<')
    attach(hairpin, target[2:])

    assert format(target) == stringtools.normalize(
        r'''
        {
            c'4 \<
            c'4 \! \<
            c'4 \! \<
            c'4 \!
        }
        '''
        )

    string = r'''\relative c' { c \< c \< c \< c \! }'''
    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__spanners__Hairpin_03():
    r'''Dynamics can terminate hairpins.
    '''

    target = Staff(scoretools.make_notes([0] * 3, [(1, 4)]))
    hairpin = Hairpin(descriptor='<')
    attach(hairpin, target[0:2])
    hairpin = Hairpin(descriptor='>')
    attach(hairpin, target[1:])
    dynamic = Dynamic('p')
    attach(dynamic, target[1])
    dynamic = Dynamic('f')
    attach(dynamic, target[-1])

    assert format(target) == stringtools.normalize(
        r'''
        \new Staff {
            c'4 \<
            c'4 \p \>
            c'4 \f
        }
        '''
        )

    string = r"\new Staff \relative c' { c \< c \p \> c \f }"
    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__spanners__Hairpin_04():
    r'''Unterminated.
    '''

    string = r'{ c \< c c c }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__Hairpin_05():
    r'''Unbegun is okay.
    '''

    string = r'{ c c c c \! }'
    result = LilyPondParser()(string)


def test_lilypondparsertools_LilyPondParser__spanners__Hairpin_06():
    r'''No double dynamic spans permitted.
    '''

    string = r'{ c \< \> c c c \! }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__Hairpin_07():
    r'''With direction.
    '''

    target = Staff(scoretools.make_notes([0] * 5, [(1, 4)]))
    hairpin = Hairpin(descriptor='<', direction=Up)
    attach(hairpin, target[:3])
    hairpin = Hairpin(descriptor='>', direction=Down)
    attach(hairpin, target[2:])
    dynamic = Dynamic('ppp')
    attach(dynamic, target[-1])

    assert format(target) == stringtools.normalize(
        r'''
        \new Staff {
            c'4 ^ \<
            c'4
            c'4 \! _ \>
            c'4
            c'4 \ppp
        }
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__spanners__Hairpin_08():

    string = r"\new Staff { c'4 ( \p \< d'4 e'4 f'4 ) \! }"
    parser = LilyPondParser()
    result = parser(string)
    assert format(result) == stringtools.normalize(
        r'''
        \new Staff {
            c'4 \p \< (
            d'4
            e'4
            f'4 \! )
        }
        '''
        )



# -*- coding: utf-8 -*-
import os
from abjad.tools import commandlinetools
from abjad.tools import systemtools
from base import ScorePackageScriptTestCase


class Test(ScorePackageScriptTestCase):

    def test_list_segments(self):
        self.create_score()
        self.create_segment('segment_one')
        self.create_segment('segment_two')
        self.create_segment('segment_three')
        script = commandlinetools.ManageSegmentScript()
        command = ['--list']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 2
        self.compare_captured_output(r'''
            Available segments:
                Reading test_score/segments/metadata.json ... OK!
                segment_one   [1]
                segment_two   [2]
                segment_three [3]
        '''.replace('/', os.path.sep))

    def test_list_segments_no_segments(self):
        self.create_score()
        script = commandlinetools.ManageSegmentScript()
        command = ['--list']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 2
        self.compare_captured_output(r'''
            Available segments:
                Reading test_score/segments/metadata.json ... JSON does not exist.
                No segments available.
        '''.replace('/', os.path.sep))

    def test_list_segments_unstaged(self):
        self.create_score()
        self.create_segment('segment_one')
        self.create_segment('segment_two')
        self.create_segment('segment_three')
        script = commandlinetools.ManageSegmentScript()
        segment_names = script._read_segments_list_json(
            self.score_path,
            verbose=False,
            )
        segment_names.remove('segment_two')
        script._write_segments_list_json(
            segment_names,
            score_path=self.score_path,
            verbose=False,
            )
        command = ['--list']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 2
        self.compare_captured_output(r'''
            Available segments:
                Reading test_score/segments/metadata.json ... OK!
                segment_one   [1]
                segment_three [2]
                segment_two
        '''.replace('/', os.path.sep))



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Skip___eq___01():

    skip_1 = scoretools.Skip((1, 4))
    skip_2 = scoretools.Skip((1, 4))
    skip_3 = scoretools.Skip((1, 8))

    assert not skip_1 == skip_2
    assert not skip_1 == skip_3
    assert not skip_2 == skip_3
